@@ -30,22 +30,29 @@
 #define FC(filters,row,col) \
     (filters >> ((((row) << 1 & 14) + ((col) & 1)) << 1) & 3)
 extern "C" {
-int  fc_INDI (const unsigned filters, const int row, const int col);
-void scale_colors_INDI(gushort (*image)[4], const int rgb_max, const int black,
+int fc_INDI (const unsigned filters, const int row, const int col);
+void wavelet_denoise_INDI(gushort (*image)[4], const int black,
+    const int iheight, const int iwidth, const int height, const int width,
+    const int colors, const int shrink, float pre_mul[4],
+    const float threshold, const unsigned filters, void *dcraw);
+void scale_colors_INDI(gushort (*image)[4], const int maximum, const int black,
     const int use_auto_wb, const int use_camera_wb, const float cam_mul[4],
-    const int height, const int width, const int colors,
-    float pre_mul[4], const unsigned filters, /*const*/ gushort white[8][8],
-    const char *ifname, void *dcraw);
+    const int iheight, const int iwidth, const int colors, float pre_mul[4],
+    const unsigned filters, /*const*/ gushort white[8][8], const char *ifname,
+    void *dcraw);
 void lin_interpolate_INDI(gushort (*image)[4], const unsigned filters,
     const int width, const int height, const int colors, void *dcraw);
 void vng_interpolate_INDI(gushort (*image)[4], const unsigned filters,
     const int width, const int height, const int colors, const int rgb_max,
     void *dcraw);
-void cam_to_cielab_INDI (gushort cam[4], float lab[3],
-    const int colors, float rgb_cam[3][4]);
 void ahd_interpolate_INDI(gushort (*image)[4], const unsigned filters,
     const int width, const int height, const int colors, float rgb_cam[3][4],
     void *dcraw);
+void eahd_interpolate_INDI(gushort (*image)[4], const unsigned filters,
+    const int width, const int height, const int colors, float rgb_cam[3][4],
+    void *dcraw);
+void ppg_interpolate_INDI(gushort (*image)[4], const unsigned filters,
+    const int width, const int height, const int colors, void *dcraw);
 void flip_image_INDI(gushort (*image)[4], int *height_p, int *width_p,
     const int flip);
 void fuji_rotate_INDI(gushort (**image_p)[4], int *height_p, int *width_p,
@@ -139,6 +146,30 @@ int dcraw_open(dcraw_data *h,char *filename)
     return d->lastStatus;
 }
 
+int dcraw_image_dimensions(dcraw_data *raw, int flip, int *height, int *width)
+{
+    if (raw->fuji_width) {
+        /* Copied from DCRaw's fuji_rotate() */
+        *width = (int)(raw->fuji_width / raw->fuji_step);
+        *height = (int)((raw->height - raw->fuji_width) / raw->fuji_step);
+    } else {
+        if (raw->pixel_aspect < 1)
+            *height = (int)(raw->height / raw->pixel_aspect + 0.5);
+        else
+            *height = raw->height;
+        if (raw->pixel_aspect > 1)
+            *width = (int)(raw->width * raw->pixel_aspect + 0.5);
+        else
+            *width = raw->width;
+    }
+    if (flip & 4) {
+        int tmp = *height;
+        *height = *width;
+        *width = tmp;
+    }
+    return DCRAW_SUCCESS;
+}
+
 int dcraw_load_raw(dcraw_data *h)
 {
     DCRaw *d = (DCRaw *)h->dcraw;
@@ -148,6 +179,12 @@ int dcraw_load_raw(dcraw_data *h)
     g_free(d->messageBuffer);
     d->messageBuffer = NULL;
     d->lastStatus = DCRAW_SUCCESS;
+    if (setjmp(d->failure)) {
+        d->dcraw_message(DCRAW_ERROR,_("Fatal internal error\n"));
+        h->message = d->messageBuffer;
+	delete d;
+        return DCRAW_ERROR;
+    }
     h->raw.height = d->iheight = (h->height+h->shrink) >> h->shrink;
     h->raw.width = d->iwidth = (h->width+h->shrink) >> h->shrink;
     h->raw.image = d->image = g_new0(dcraw_image_type, d->iheight * d->iwidth
@@ -170,8 +207,8 @@ int dcraw_load_raw(dcraw_data *h)
     d->bad_pixels();
     if (d->is_foveon) {
         d->foveon_interpolate();
-	h->raw.width = d->width;
-	h->raw.height = d->height;
+	h->raw.width = h->width = d->width;
+	h->raw.height = h->height = d->height;
     }
     fclose(d->ifp);
     h->ifp = NULL;
@@ -179,7 +216,6 @@ int dcraw_load_raw(dcraw_data *h)
     h->black = d->black;
     d->dcraw_message(DCRAW_VERBOSE,_("Black: %d, Maximum: %d\n"),
 	    d->black, d->maximum);
-    cam_to_cielab_INDI(NULL, NULL, h->colors, d->rgb_cam);
     dmin = DBL_MAX;
     for (i=0; i<h->colors; i++) if (dmin > d->pre_mul[i]) dmin = d->pre_mul[i];
     for (i=0; i<h->colors; i++) h->pre_mul[i] = d->pre_mul[i]/dmin;
@@ -406,6 +442,35 @@ int dcraw_set_color_scale(dcraw_data *h, int useAutoWB, int useCameraWB)
     return d->lastStatus;
 }
 
+int dcraw_wavelet_denoise(dcraw_data *h, float threshold)
+{
+    DCRaw *d = (DCRaw *)h->dcraw;
+    g_free(d->messageBuffer);
+    d->messageBuffer = NULL;
+    d->lastStatus = DCRAW_SUCCESS;
+    memcpy(h->post_mul, h->pre_mul, sizeof h->post_mul);
+    if (threshold)
+	wavelet_denoise_INDI(h->raw.image, h->black, h->raw.height,
+		h->raw.width, h->height, h->width, h->raw.colors, h->shrink,
+		h->post_mul, threshold, h->filters, d);
+    h->message = d->messageBuffer;
+    return d->lastStatus;
+}
+
+int dcraw_wavelet_denoise_shrinked(dcraw_image_data *f,
+	dcraw_data *h, float threshold)
+{
+    DCRaw *d = (DCRaw *)h->dcraw;
+    g_free(d->messageBuffer);
+    d->messageBuffer = NULL;
+    d->lastStatus = DCRAW_SUCCESS;
+    if (threshold)
+	wavelet_denoise_INDI(f->image, 0, f->height, f->width, 0, 0, 4, 0,
+		NULL, threshold, 0, d);
+    h->message = d->messageBuffer;
+    return d->lastStatus;
+}
+
 int dcraw_finalize_interpolate(dcraw_image_data *f, dcraw_data *h,
 	int interpolation, int rgbWB[4])
 {
@@ -436,7 +501,10 @@ int dcraw_finalize_interpolate(dcraw_image_data *f, dcraw_data *h,
     }
     /* It might be better to report an error here: */
     /* (dcraw also forbids AHD for Fuji rotated images) */
-    if (interpolation==dcraw_ahd_interpolation && h->colors > 3)
+    if ((interpolation==dcraw_ahd_interpolation ||
+	 interpolation==dcraw_eahd_interpolation) && h->colors > 3)
+	interpolation = dcraw_vng_interpolation;
+    if (interpolation==dcraw_ppg_interpolation && h->colors > 3)
 	interpolation = dcraw_vng_interpolation;
     f4 = h->fourColorFilters;
     if (h->colors==3) rgbWB[3] = rgbWB[1];
@@ -453,6 +521,11 @@ int dcraw_finalize_interpolate(dcraw_image_data *f, dcraw_data *h,
     else if (interpolation==dcraw_ahd_interpolation)
 	ahd_interpolate_INDI(f->image, ff, f->width, f->height, cl,
 		h->rgb_cam, d);
+    else if (interpolation==dcraw_eahd_interpolation)
+	eahd_interpolate_INDI(f->image, ff, f->width, f->height, cl,
+		h->rgb_cam, d);
+    else if (interpolation==dcraw_ppg_interpolation)
+	ppg_interpolate_INDI(f->image, ff, f->width, f->height, cl, d);
 
     if (cl==4 && h->colors == 3) {
         for (i=0; i<f->height*f->width; i++)
