@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * dcraw_api.h - API for DCRaw
- * Copyright 2004-2006 by Udi Fuchs
+ * Copyright 2004-2007 by Udi Fuchs
  *
  * based on dcraw by Dave Coffin
  * http://www.cybercom.net/~dcoffin/
@@ -47,8 +47,9 @@ typedef struct {
     int thumbType, thumbOffset, thumbBufferLength;
 } dcraw_data;
 
-enum { dcraw_ahd_interpolation, dcraw_vng_interpolation,
-    dcraw_four_color_interpolation, dcraw_bilinear_interpolation };
+enum { dcraw_eahd_interpolation, dcraw_ahd_interpolation,
+    dcraw_vng_interpolation, dcraw_four_color_interpolation,
+    dcraw_ppg_interpolation, dcraw_bilinear_interpolation };
 enum { unknown_thumb_type, jpeg_thumb_type, ppm_thumb_type };
 int dcraw_open(dcraw_data *h, char *filename);
 int dcraw_load_raw(dcraw_data *h);
@@ -58,16 +59,21 @@ int dcraw_image_resize(dcraw_image_data *image, int size);
 int dcraw_image_stretch(dcraw_image_data *image, double pixel_aspect);
 int dcraw_flip_image(dcraw_image_data *image, int flip);
 int dcraw_set_color_scale(dcraw_data *h, int useAutoWB, int useCameraWB);
+int dcraw_wavelet_denoise(dcraw_data *h, float threshold);
+int dcraw_wavelet_denoise_shrinked(dcraw_image_data *f,
+        dcraw_data *h, float threshold);
 int dcraw_finalize_interpolate(dcraw_image_data *f, dcraw_data *h,
 	int interpolation, int rgbWB[4]);
 void dcraw_close(dcraw_data *h);
+int dcraw_image_dimensions(dcraw_data *raw, int flip, int *height, int *width);
 
 #define DCRAW_SUCCESS 0
 #define DCRAW_ERROR 1
 #define DCRAW_UNSUPPORTED 2
 #define DCRAW_NO_CAMERA_WB 3
 #define DCRAW_VERBOSE 4
-#define DCRAW_OPEN_ERROR 5
+#define DCRAW_WARNING 5
+#define DCRAW_OPEN_ERROR 6
 
 void dcraw_message(void *dcraw, int code, char *format, ...);
 
