@@ -1,7 +1,7 @@
 /***************************************************
  nikon_curve.c - read Nikon NTC/NCV files
  
- Copyright 2004-2006 by Shawn Freeman, Udi Fuchs
+ Copyright 2004-2007 by Shawn Freeman, Udi Fuchs
 
  This program reads in a Nikon NTC/NCV file, 
  interperates it's tone curve, and writes out a
@@ -211,8 +211,7 @@ void nc_message(int code, char *format, ...)
 #else    //else, just print out the errors normally
 
     code = code;
-    fprintf(stderr, message);
-    fflush(stderr);
+    g_printerr("%s", message);
 
 #endif //End WITH_UFRAW
 
@@ -232,12 +231,30 @@ void DEBUG_PRINT(char *format, ...)
 #endif
 }
 
+/* nc_merror(): Handle memory allocaltion errors */
+void nc_merror(void *ptr, char *where)
+{
+    if (ptr) return;
+#ifdef __WITH_UFRAW__
+    g_error("Out of memory in %s\n", where);
+#else
+    fprintf(stderr, "Out of memory in %s\n", where);
+    exit(1);
+#endif
+}
+
+// Assert something at compile time (must use this inside a function);
+// works because compilers won't let us declare negative-length arrays.
+#define STATIC_ASSERT(cond) \
+    { (void)((int (*)(char failed_static_assertion[(cond)?1:-1]))0); }
+
 /***********************************************************************
 isBigEndian:
 	Determines if the machine we are running on is big endian or not.
 ************************************************************************/
 int isBigEndian()
 {
+    STATIC_ASSERT(sizeof(short)==2);
     short x;
     unsigned char EndianTest[2] = { 1, 0 };
 
@@ -252,6 +269,7 @@ ShortVal:
 ************************************************************************/
 short ShortVal(short s)
 {
+    STATIC_ASSERT(sizeof(short)==2);
     if (isBigEndian()) {
 	unsigned char b1, b2;
 
@@ -269,6 +287,7 @@ LongVal:
 ************************************************************************/
 int LongVal(int i)
 {
+    STATIC_ASSERT(sizeof(int)==4);
     if (isBigEndian()) {
 	unsigned char b1, b2, b3, b4;
 
@@ -288,6 +307,7 @@ FloatVal:
 ************************************************************************/
 float FloatVal(float f)
 {
+    STATIC_ASSERT(sizeof(float)==4);
     if (isBigEndian()) {
 	union {
 	    float f;
@@ -310,6 +330,7 @@ DoubleVal:
 ************************************************************************/
 double DoubleVal(double d)
 {
+    STATIC_ASSERT(sizeof(double)==8);
     if (isBigEndian()) {
 	union {
 	    double d;
@@ -396,6 +417,7 @@ double *d3_np_fs ( int n, double a[], double b[] )
     }
   }
   x = (double *)calloc(n,sizeof(double));
+  nc_merror(x, "d3_np_fs");
 
   for ( i = 0; i < n; i++ )
   {
@@ -557,7 +579,9 @@ double *spline_cubic_set ( int n, double t[], double y[], int ibcbeg, double ybc
     }
   }
   a = (double *)calloc(3*n,sizeof(double));
+  nc_merror(a, "spline_cubic_set");
   b = (double *)calloc(n,sizeof(double));
+  nc_merror(b, "spline_cubic_set");
 //
 //  Set up the first equation.
 //
@@ -633,6 +657,7 @@ double *spline_cubic_set ( int n, double t[], double y[], int ibcbeg, double ybc
   if ( n == 2 && ibcbeg == 0 && ibcend == 0 )
   {
     ypp = (double *)calloc(2,sizeof(double));
+    nc_merror(ypp, "spline_cubic_set");
 
     ypp[0] = 0.0E+00;
     ypp[1] = 0.0E+00;
@@ -1030,6 +1055,7 @@ int CurveDataSample(CurveData *curve, CurveSample *sample)
     
     sample->m_Samples = (unsigned int *)realloc(sample->m_Samples,
 	    sample->m_samplingRes * sizeof(int));
+    nc_merror(sample->m_Samples, "CurveDataSample");
 
     int firstPointX = x[0] * (sample->m_samplingRes-1);
     int firstPointY = pow(y[0], gamma) * (sample->m_outputRes-1);
@@ -1652,10 +1678,12 @@ CurveSampleInit:
 CurveSample *CurveSampleInit(unsigned int samplingRes, unsigned int outputRes)
 {
     CurveSample *sample = (CurveSample*)calloc(1, sizeof(CurveSample));
+    nc_merror(sample, "CurveSampleInit");
     sample->m_samplingRes = samplingRes;
     sample->m_outputRes = outputRes;
     if (samplingRes>0) {
 	sample->m_Samples = (unsigned int*)calloc(samplingRes, sizeof(int));
+	nc_merror(sample->m_Samples, "CurveSampleInit");
     } else {
 	sample->m_Samples = NULL;
     }
