@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw-batch.c - The standalone interface to UFRaw in batch mode.
- * Copyright 2004-2006 by Udi Fuchs
+ * Copyright 2004-2007 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2
@@ -77,10 +77,16 @@ int main (int argc, char **argv)
             continue;
         }
         status = ufraw_config(uf, &rc, &conf, &cmd);
-	if (status==UFRAW_ERROR) exit(1);
-
-        if (ufraw_load_raw(uf)!=UFRAW_SUCCESS)
+	if (status==UFRAW_ERROR) {
+	    ufraw_close(uf);
+	    g_free(uf);
+	    exit(1);
+	}
+        if (ufraw_load_raw(uf)!=UFRAW_SUCCESS) {
+	    ufraw_close(uf);
+	    g_free(uf);
             continue;
+	}
         ufraw_message(UFRAW_MESSAGE, _("loaded %s"), uf->filename);
         if (ufraw_batch_saver(uf)==UFRAW_SUCCESS)
             ufraw_message(UFRAW_MESSAGE, _("saved %s"),
