@@ -2,7 +2,7 @@
  * UFRaw - Unidentified Flying Raw converter for digital camera images
  *
  * ufraw.c - The standalone interface to UFRaw.
- * Copyright 2004-2006 by Udi Fuchs
+ * Copyright 2004-2007 by Udi Fuchs
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2
@@ -38,12 +38,12 @@ int main (int argc, char **argv)
     ufraw_icons_init();
 #ifdef WIN32
     dummyWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-#ifdef HAVE_GTK_2_6
-    gtk_window_set_icon_name(GTK_WINDOW(dummyWindow), "ufraw-ufraw");
+#if GTK_CHECK_VERSION(2,6,0)
+    gtk_window_set_icon_name(GTK_WINDOW(dummyWindow), "ufraw");
 #else
     gtk_window_set_icon(GTK_WINDOW(dummyWindow),
 	    gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
-		    "ufraw-ufraw", 48, GTK_ICON_LOOKUP_USE_BUILTIN, NULL));
+		    "ufraw", 48, GTK_ICON_LOOKUP_USE_BUILTIN, NULL));
 #endif
     ufraw_message(UFRAW_SET_PARENT, (char *)dummyWindow);
 #endif
@@ -80,12 +80,12 @@ int main (int argc, char **argv)
     /* Create a dummyWindow for the GUI error messenger */
     if (dummyWindow==NULL) {
 	dummyWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-#ifdef HAVE_GTK_2_6
-	gtk_window_set_icon_name(GTK_WINDOW(dummyWindow), "ufraw-ufraw");
+#if GTK_CHECK_VERSION(2,6,0)
+	gtk_window_set_icon_name(GTK_WINDOW(dummyWindow), "ufraw");
 #else
 	gtk_window_set_icon(GTK_WINDOW(dummyWindow),
 		gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
-			"ufraw-ufraw", 48, GTK_ICON_LOOKUP_USE_BUILTIN, NULL));
+			"ufraw", 48, GTK_ICON_LOOKUP_USE_BUILTIN, NULL));
 #endif
 	ufraw_message(UFRAW_SET_PARENT, (char *)dummyWindow);
     }
@@ -126,9 +126,14 @@ int main (int argc, char **argv)
             continue;
         }
         status = ufraw_config(uf, &rc, &conf, &cmd);
-	if (status==UFRAW_ERROR) exit(1);
+	if (status==UFRAW_ERROR) {
+	    ufraw_close(uf);
+	    g_free(uf);
+	    exit(1);
+	}
         ufraw_preview(uf, FALSE, ufraw_saver);
 	rc = *uf->conf;
+	g_free(uf);
     }
     if (dummyWindow!=NULL) gtk_widget_destroy(dummyWindow);
 
