@@ -41,18 +41,26 @@ enum { rgb_histogram, r_g_b_histogram, luminosity_histogram, value_histogram,
 enum { linear_histogram, log_histogram };
 /* The following enum should match the dcraw_interpolation enum
  * in dcraw_api.h. */
-enum { ahd_interpolation, vng_interpolation, four_color_interpolation,
-       bilinear_interpolation, half_interpolation };
+enum { eahd_interpolation, ahd_interpolation,
+       vng_interpolation, four_color_interpolation,
+       ppg_interpolation, bilinear_interpolation, half_interpolation,
+       num_interpolations };
 extern const char *interpolationNames[];
 enum { no_id, also_id, only_id };
 enum { manual_curve, linear_curve, custom_curve, camera_curve };
-enum { in_profile, out_profile, profile_types};
+enum { in_profile, out_profile, display_profile, profile_types};
 enum { raw_expander, live_expander, expander_count };
 enum { ppm8_type, ppm16_type, tiff8_type, tiff16_type, jpeg_type,
        embedded_jpeg_type, embedded_png_type };
 enum { clip_details, restore_lch_details, restore_hsv_details,
        restore_types };
 enum { digital_highlights, film_highlights, highlights_types };
+typedef enum { display_developer, file_developer, auto_developer }
+	DeveloperMode;
+typedef enum { perceptual_intent, relative_intent, saturation_intent,
+	absolute_intent, disable_intent } Intent;
+typedef enum { ufraw_first_phase, ufraw_denoise_phase, ufraw_final_phase,
+	ufraw_phases_num } UFRawPhase;
 
 typedef struct {
     char *make;
@@ -67,9 +75,9 @@ typedef struct {
     int restoreDetails, clipHighlights;
     int rgbWB[4], colorMatrix[3][4];
     double gamma, linear;
-    char profileFile[2][max_path];
-    void *profile[2];
-    int intent;
+    char profileFile[profile_types][max_path];
+    void *profile[profile_types];
+    Intent intent[profile_types];
     gboolean updateTransform;
     void *colorTransform;
     double saturation;
@@ -128,6 +136,7 @@ typedef struct {
     double WBTuning;
     double temperature, green;
     double chanMul[4];
+    double threshold;
     double exposure, saturation, black; /* black is only used in CMD */
     int ExposureNorm;
     int restoreDetails, clipHighlights;
@@ -138,10 +147,11 @@ typedef struct {
     CurveData curve[max_curves];
     int profileIndex[profile_types], profileCount[profile_types];
     profile_data profile[profile_types][max_profiles];
-    int intent;
+    Intent intent[profile_types];
     int interpolation;
     char darkframeFile[max_path];
     struct ufraw_struct *darkframe;
+    int CropX1, CropY1, CropX2, CropY2;
 
     /* SAVE options */
     char inputFilename[max_path], outputFilename[max_path],
@@ -164,7 +174,7 @@ typedef struct {
     gboolean silent;
 
     /* EXIF data */
-    int flip;
+    int orientation;
     float iso_speed, shutter, aperture, focal_len;
     char exifSource[max_name], isoText[max_name], shutterText[max_name],
 	 apertureText[max_name], focalLenText[max_name],
@@ -172,20 +182,31 @@ typedef struct {
     char timestamp[max_name], make[max_name], model[max_name];
 } conf_data;
 
+typedef struct {
+    guint8 *buffer;
+    int height, width, depth, rowstride;
+} ufraw_image_data;
+
+/* image_data should be phased out and replaced by ufraw_image_data */
 typedef struct {
     image_type *image;
     guint8 *buffer;
-    int height, width;
+    int height, width, depth, rowstride;
 } image_data;
 
 typedef struct ufraw_struct {
     char filename[max_path];
-    int predictedHeight, predictedWidth, rgbMax, colors, raw_color, useMatrix;
+    int initialHeight, initialWidth, rgbMax, colors, raw_color, useMatrix;
     gboolean LoadingID; /* Indication that we are loading an ID file */
     float rgb_cam[3][4];
+    int ConvertShrink;
+    ufraw_image_data Images[ufraw_phases_num];
     image_data image;
     image_data thumb;
     void *raw;
+    gboolean HaveFilters;
+    void *unzippedBuf;
+    unsigned unzippedBufLen;
     developer_data *developer;
     conf_data *conf;
     void *widget;
@@ -213,7 +234,13 @@ int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf,conf_data *cmd);
 int ufraw_load_raw(ufraw_data *uf);
 ufraw_data *ufraw_load_darkframe(char *darkframeFilename);
 int ufraw_convert_image(ufraw_data *uf);
+int ufraw_convert_image_init(ufraw_data *uf);
+int ufraw_convert_image_first_phase(ufraw_data *uf);
+int ufraw_convert_image_init_phase(ufraw_data *uf);
+int ufraw_convert_image_area(ufraw_data *uf,
+	int x, int y, int width, int height, UFRawPhase fromPhase);
 void ufraw_close(ufraw_data *uf);
+int ufraw_flip_image(ufraw_data *uf, int flip);
 int ufraw_set_wb(ufraw_data *uf);
 void ufraw_auto_expose(ufraw_data *uf);
 void ufraw_auto_black(ufraw_data *uf);
@@ -258,9 +285,10 @@ void developer_destroy(developer_data *d);
 void developer_profile(developer_data *d, int type, profile_data *p);
 void developer_prepare(developer_data *d, conf_data *conf,
     int rgbMax, float rgb_cam[3][4], int colors, int useMatrix,
-    gboolean autoTools);
+    DeveloperMode mode);
 void develope(void *po, guint16 pix[4], developer_data *d, int mode,
     guint16 *buf, int count);
+void develop_linear(guint16 in[4], guint16 out[3], developer_data *d);
 
 /* prototype for functions in ufraw_saver.c */
 long ufraw_saver(void *widget, gpointer user_data);
@@ -280,8 +308,8 @@ void ufraw_chooser(conf_data *conf, char *defPath);
 /* prototype for functions in ufraw_icons.c */
 void ufraw_icons_init();
 
-/* prototype for functions in ufraw_exif.c */
-int ufraw_exif_from_raw(ufraw_data *uf);
+/* prototype for functions in ufraw_exiv2.cc */
+int ufraw_exif_from_exiv2(ufraw_data *uf);
 
 /* status numbers from DCRaw and UFRaw */
 #define UFRAW_SUCCESS 0
@@ -293,7 +321,6 @@ int ufraw_exif_from_raw(ufraw_data *uf);
 #define UFRAW_DCRAW_SET_LOG 4 /* DCRAW_VERBOSE */
 #define UFRAW_ERROR 100
 #define UFRAW_CANCEL 101
-#define UFRAW_ABORT_SAVE 102 /* Unrecoverable error during image save */
 #define UFRAW_RC_VERSION 103 /* Mismatch in version from .ufrawrc */
 #define UFRAW_WARNING 104
 #define UFRAW_MESSAGE 105
@@ -309,9 +336,6 @@ int ufraw_exif_from_raw(ufraw_data *uf);
 #define UFRAW_CLEAN 209 /* Clean all buffers */
 #define UFRAW_RESET 210 /* Reset warnings and errors */
 #define UFRAW_SET_PARENT 211 /* Set parent window for message dialog */
-#define UFRAW_LCMS_WARNING 0x1000
-#define UFRAW_LCMS_RECOVERABLE 0x2000
-#define UFRAW_LCMS_ABORTED 0x3000
 
 char *ufraw_message(int code, const char *format, ...);
 
