@@ -23,6 +23,10 @@
 #include <glib/gi18n.h>
 #include "ufraw.h"
 
+#ifdef HAVE_EXIV2
+#include <exiv2/exv_conf.h>
+#endif
+
 const conf_data conf_default = {
     /* Internal data */
     sizeof(conf_data), 7, /* confSize, version */
@@ -31,6 +35,7 @@ const conf_data conf_default = {
     camera_wb, 0, /* wb, WBTuning */
     6500, 1.0, /* temperature, green */
     { -1.0, -1.0, -1.0, -1.0 }, /* chanMul[] */
+    0.0, /* wavelet denoising threshold */
     0.0, 1.0, 0.0, /* exposure, saturation, black */
     0, /* ExposureNorm */
     restore_lch_details, /* restoreDetails */
@@ -55,15 +60,18 @@ const conf_data conf_default = {
       { N_("Linear curve"), TONE_CURVE, 0.0, 1.0, 0.0, 1.0, 1.0,
 	  2 , { { 0.0, 0.0 }, { 1.0, 1.0 } } }
     },
-    { 0, 0 } , { 1, 1 }, /* profileIndex[], profileCount[] */
+    { 0, 0, 0 } , { 1, 1, 1 }, /* profileIndex[], profileCount[] */
     /* Profile data defaults */
     { { { N_("sRGB"), "", "", 0.45, 0.1, TRUE },
         { "Some ICC Profile", "", "", 0.45, 0.0, FALSE } },
+      { { N_("sRGB"), "", "", 0.0, 0.0, FALSE },
+        { "Some ICC Profile", "", "", 0.0, 0.0, FALSE } },
       { { N_("sRGB"), "", "", 0.0, 0.0, FALSE },
         { "Some ICC Profile", "", "", 0.0, 0.0, FALSE } } },
-    0, /* intent */
+    { 0, 0, 0 }, /* intent */
     ahd_interpolation, /* interpolation */
     "", NULL, /* darkframeFile, darkframe */
+    -1, -1, -1, -1, /* Crop X1,Y1,X2,Y2 */
     /* Save options */
     "", "", "", /* inputFilename, outputFilename, outputPath */
     "", "", /* inputURI, inputModTime */
@@ -87,7 +95,7 @@ const conf_data conf_default = {
     FALSE, /* silent */
 
     /* EXIF data */
-    0, /* flip */
+    -1, /* orientation */
     0.0, 0.0, 0.0, 0.0, /* iso_speed, shutter, aperture, focal_len */
     "", "", "", "", /* exifSource, isoText, shutterText, apertureText */
     "", "", "", /* focalLenText, focalLen35Text, lensText */
@@ -95,11 +103,13 @@ const conf_data conf_default = {
 };
 
 const char *interpolationNames[] =
-    { "ahd", "vng", "four-color", "bilinear", "half", NULL };
+    { "eahd", "ahd", "vng", "four-color", "ppg", "bilinear", "half", NULL };
 const char *restoreDetailsNames[] =
     { "clip", "lch", "hsv", NULL };
 const char *clipHighlightsNames[] =
     { "digital", "film", NULL };
+const char *intentNames[] =
+    { "perceptual", "relative", "saturation", "absolute", "disable", NULL };
 
 int conf_find_name(const char name[],const char *namesList[], int notFound)
 {
@@ -167,13 +177,18 @@ void conf_parse_start(GMarkupParseContext *context, const gchar *element,
             if (!strcmp("Curve", element))
                 c->curveIndex = c->curveCount;
             if (!strcmp("sRGBInputProfile", element))
-                c->profileIndex[0] = 0;
+                c->profileIndex[in_profile] = 0;
             if (!strcmp("sRGBOutputProfile", element))
-                c->profileIndex[1] = 0;
+                c->profileIndex[out_profile] = 0;
+            if (!strcmp("sRGBDisplayProfile", element))
+                c->profileIndex[display_profile] = 0;
             if (!strcmp("InputProfile", element))
-                c->profileIndex[0] = c->profileCount[0];
+                c->profileIndex[in_profile] = c->profileCount[in_profile];
             if (!strcmp("OutputProfile", element))
-                c->profileIndex[1] = c->profileCount[1];
+                c->profileIndex[out_profile] = c->profileCount[out_profile];
+            if (!strcmp("DisplayProfile", element))
+                c->profileIndex[display_profile] =
+			c->profileCount[display_profile];
         }
         names++;
         values++;
@@ -205,8 +220,12 @@ void conf_parse_start(GMarkupParseContext *context, const gchar *element,
 	c->curveCount = - linear_curve;
 	c->curve[-c->curveCount].m_numAnchors = 0;
     }
-    if (!strcmp("sRGBInputProfile", element)) c->profileCount[0] = - 0;
-    if (!strcmp("sRGBOutputProfile", element)) c->profileCount[1] = - 0;
+    if ( !strcmp("sRGBInputProfile", element) )
+	c->profileCount[in_profile] = - 0;
+    if ( !strcmp("sRGBOutputProfile", element) )
+	c->profileCount[out_profile] = - 0;
+    if ( !strcmp("sRGBDisplayProfile", element) )
+	c->profileCount[display_profile] = - 0;
 }
 
 void conf_parse_end(GMarkupParseContext *context, const gchar *element,
@@ -241,12 +260,20 @@ void conf_parse_end(GMarkupParseContext *context, const gchar *element,
 	    c->curve[-c->curveCount].m_numAnchors = 2;
         c->curveCount = - c->curveCount + 1;
     }
-    if (!strcmp("sRGBInputProfile", element)) c->profileCount[0] = 1;
-    if (!strcmp("sRGBOutputProfile", element)) c->profileCount[1] = 1;
-    if (c->profileCount[0]<=0 && !strcmp("InputProfile", element))
-        c->profileCount[0] = - c->profileCount[0] + 1;
-    if (c->profileCount[1]<=0 && !strcmp("OutputProfile", element))
-        c->profileCount[1] = - c->profileCount[1] + 1;
+    if ( !strcmp("sRGBInputProfile", element) )
+	c->profileCount[in_profile] = 1;
+    if ( !strcmp("sRGBOutputProfile", element) )
+	c->profileCount[out_profile] = 1;
+    if ( !strcmp("sRGBDisplayProfile", element) )
+	c->profileCount[display_profile] = 1;
+    if ( c->profileCount[in_profile]<=0 && strcmp("InputProfile", element)==0 )
+        c->profileCount[in_profile] = -c->profileCount[in_profile]+1;
+    if ( c->profileCount[out_profile]<=0 &&
+	 strcmp("OutputProfile", element)==0 )
+        c->profileCount[out_profile] = - c->profileCount[out_profile] + 1;
+    if ( c->profileCount[display_profile]<=0 &&
+	 strcmp("DisplayProfile", element)==0 )
+        c->profileCount[display_profile] = -c->profileCount[display_profile]+1;
 }
 
 void conf_parse_text(GMarkupParseContext *context, const gchar *text, gsize len,
@@ -307,30 +334,43 @@ void conf_parse_text(GMarkupParseContext *context, const gchar *text, gsize len,
         return;
     }
 
-    if (c->profileCount[0]<=0) {
-        i = - c->profileCount[0];
+    if (c->profileCount[in_profile]<=0) {
+        i = - c->profileCount[in_profile];
         if (!strcmp("File", element))
-            g_strlcpy(c->profile[0][i].file, temp, max_path);
+            g_strlcpy(c->profile[in_profile][i].file, temp, max_path);
         if (!strcmp("ProductName", element))
-            g_strlcpy(c->profile[0][i].productName, temp, max_path);
+            g_strlcpy(c->profile[in_profile][i].productName, temp, max_name);
         if (!strcmp("Gamma", element))
-            sscanf(temp, "%lf", &c->profile[0][i].gamma);
+            sscanf(temp, "%lf", &c->profile[in_profile][i].gamma);
         if (!strcmp("Linearity", element))
-            sscanf(temp, "%lf", &c->profile[0][i].linear);
+            sscanf(temp, "%lf", &c->profile[in_profile][i].linear);
         if (!strcmp("UseColorMatrix", element))
-            sscanf(temp, "%d", &c->profile[0][i].useMatrix);
+            sscanf(temp, "%d", &c->profile[in_profile][i].useMatrix);
         return;
     }
-    if (c->profileCount[1]<=0) {
-        i = - c->profileCount[1];
+    if (c->profileCount[out_profile]<=0) {
+        i = - c->profileCount[out_profile];
         if (!strcmp("File", element)) {
 	    char *utf8 = g_filename_from_utf8(temp, -1, NULL, NULL, NULL);
 	    if (utf8==NULL) utf8 = g_strdup("Unknown file name");
-            g_strlcpy(c->profile[1][i].file, utf8, max_path);
+            g_strlcpy(c->profile[out_profile][i].file, utf8, max_path);
 	    g_free(utf8);
 	}
         if (!strcmp("ProductName", element))
-            g_strlcpy(c->profile[0][i].productName, temp, max_path);
+            g_strlcpy(c->profile[out_profile][i].productName, temp, max_path);
+        return;
+    }
+    if (c->profileCount[display_profile]<=0) {
+        i = - c->profileCount[display_profile];
+        if (!strcmp("File", element)) {
+	    char *utf8 = g_filename_from_utf8(temp, -1, NULL, NULL, NULL);
+	    if (utf8==NULL) utf8 = g_strdup("Unknown file name");
+            g_strlcpy(c->profile[display_profile][i].file, utf8, max_path);
+	    g_free(utf8);
+	}
+        if (!strcmp("ProductName", element))
+            g_strlcpy(c->profile[display_profile][i].productName, temp,
+		    max_path);
         return;
     }
     if (!strcmp("BaseCurve", element)) {
@@ -350,16 +390,23 @@ void conf_parse_text(GMarkupParseContext *context, const gchar *text, gsize len,
 	c->curve[-c->curveCount].m_numAnchors = 0;
     }
     if (!strcmp("InputProfile", element)) {
-        c->profileCount[0] = - c->profileCount[0];
-        i = - c->profileCount[0];
-        c->profile[0][i] = conf_default.profile[0][1];
-        g_strlcpy(c->profile[0][i].name, temp, max_name);
+        c->profileCount[in_profile] = - c->profileCount[in_profile];
+        i = - c->profileCount[in_profile];
+        c->profile[in_profile][i] = conf_default.profile[in_profile][1];
+        g_strlcpy(c->profile[in_profile][i].name, temp, max_name);
     }
     if (!strcmp("OutputProfile", element)) {
-        c->profileCount[1] = - c->profileCount[1];
-        i = - c->profileCount[1];
-        c->profile[1][i] = conf_default.profile[1][0];
-        g_strlcpy(c->profile[1][i].name, temp, max_name);
+        c->profileCount[out_profile] = - c->profileCount[out_profile];
+        i = - c->profileCount[out_profile];
+        c->profile[out_profile][i] = conf_default.profile[out_profile][0];
+        g_strlcpy(c->profile[out_profile][i].name, temp, max_name);
+    }
+    if (!strcmp("DisplayProfile", element)) {
+        c->profileCount[display_profile] = - c->profileCount[display_profile];
+        i = - c->profileCount[display_profile];
+        c->profile[display_profile][i] =
+		conf_default.profile[display_profile][0];
+        g_strlcpy(c->profile[display_profile][i].name, temp, max_name);
     }
     if (!strcmp("InputFilename", element)) {
 	char *utf8 = g_filename_from_utf8(temp, -1, NULL, NULL, NULL);
@@ -439,6 +486,8 @@ void conf_parse_text(GMarkupParseContext *context, const gchar *text, gsize len,
 		&c->chanMul[0], &c->chanMul[1], &c->chanMul[2], &c->chanMul[3]);
 	if (i<4) c->chanMul[3] = 0.0;
     }
+    if (!strcmp("WaveletDenoisingThreshold", element))
+	sscanf(temp, "%lf", &c->threshold);
     if (!strcmp("Exposure", element)) sscanf(temp, "%lf", &c->exposure);
     if (!strcmp("ExposureNorm", element)) sscanf(temp, "%d", &c->ExposureNorm);
     if (!strcmp("Saturation", element)) sscanf(temp, "%lf", &c->saturation);
@@ -463,7 +512,19 @@ void conf_parse_text(GMarkupParseContext *context, const gchar *text, gsize len,
             g_strlcpy(c->curvePath, utf8, max_path);
 	g_free(utf8);
     }
-    if (!strcmp("Intent", element)) sscanf(temp, "%d", &c->intent);
+    if ( strcmp("Intent", element)==0 ) {
+	/* Keep compatibility with numbers from ufraw-0.11 */
+        if (sscanf(temp, "%d", &i)==1) c->intent[out_profile] = i;
+	else c->intent[out_profile] = conf_find_name(temp, intentNames,
+		    conf_default.intent[out_profile]);
+    }
+    /* OutputIntent replaces Intent starting from ufraw-0.12. */
+    if ( strcmp("OutputIntent", element)==0 )
+	c->intent[out_profile] = conf_find_name(temp, intentNames,
+		    conf_default.intent[out_profile]);
+    if ( strcmp("DisplayIntent", element)==0 )
+	c->intent[display_profile] = conf_find_name(temp, intentNames,
+		conf_default.intent[display_profile]);
     if (!strcmp("Make", element)) g_strlcpy(c->make, temp, max_name);
     if (!strcmp("Model", element)) g_strlcpy(c->model, temp, max_name);
     if (!strcmp("DarkframeFile", element))
@@ -474,6 +535,9 @@ void conf_parse_text(GMarkupParseContext *context, const gchar *text, gsize len,
             g_strlcpy(c->profilePath, utf8, max_path);
 	g_free(utf8);
     }
+    if (!strcmp("Orientation", element)) sscanf(temp, "%d", &c->orientation);
+    if (!strcmp("Crop", element)) sscanf(temp, "%d %d %d %d",
+	    &c->CropX1, &c->CropY1, &c->CropX2, &c->CropY2);
     if (!strcmp("Shrink", element)) sscanf(temp, "%d", &c->shrink);
     if (!strcmp("Size", element)) sscanf(temp, "%d", &c->size);
     if (!strcmp("OutputType", element)) sscanf(temp, "%d", &c->type);
@@ -549,8 +613,8 @@ int conf_load(conf_data *c, const char *IDFilename)
     if (c->version==3) {
         c->version = conf_default.version;
         /* Don't add linear part to existing profile curves (except sRGB) */
-        for (i=2; i<c->profileCount[0]; i++)
-            c->profile[0][i].linear = 0.0;
+        for (i=2; i<c->profileCount[in_profile]; i++)
+            c->profile[in_profile][i].linear = 0.0;
     }
     if (c->version==5) {
         c->version = conf_default.version;
@@ -600,7 +664,7 @@ int conf_save(conf_data *c, char *IDFilename, char **confBuffer)
 	g_free(outPath);
 	g_free(inPath);
     }
-					    
+
     /* GUI settings are only saved to .ufrawrc */
     if (IDFilename==NULL) {
         if (c->saveConfiguration!=conf_default.saveConfiguration)
@@ -675,6 +739,10 @@ int conf_save(conf_data *c, char *IDFilename, char **confBuffer)
 		    c->chanMul[0], c->chanMul[1], c->chanMul[2], c->chanMul[3]);
 	}
     }
+    if (c->threshold!=conf_default.threshold)
+        buf = uf_markup_buf(buf,
+		"<WaveletDenoisingThreshold>%d</WaveletDenoisingThreshold>\n",
+		(int)floor(c->threshold));
     if (c->exposure!=conf_default.exposure)
         buf = uf_markup_buf(buf, "<Exposure>%lf</Exposure>\n", c->exposure);
     if (c->ExposureNorm!=conf_default.ExposureNorm)
@@ -787,7 +855,8 @@ int conf_save(conf_data *c, char *IDFilename, char **confBuffer)
     }
     for (j=0; j<profile_types; j++) {
         type = j==in_profile ? "InputProfile" :
-                j==out_profile ? "OutputProfile" : "Error";
+                j==out_profile ? "OutputProfile" :
+                j==display_profile ? "DisplayProfile" : "Error";
 	/* The default sRGB profile is conf_default.profile[j][0] */
         if ( c->profileIndex[j]==0 || ( IDFilename==NULL &&
              ( c->profile[j][0].gamma!=conf_default.profile[0][0].gamma ||
@@ -837,8 +906,12 @@ int conf_save(conf_data *c, char *IDFilename, char **confBuffer)
             buf = uf_markup_buf(buf, "</%s>\n", type);
         }
     }
-    if (c->intent!=conf_default.intent)
-        buf = uf_markup_buf(buf, "<Intent>%d</Intent>\n", c->intent);
+    if (c->intent[out_profile]!=conf_default.intent[out_profile])
+        buf = uf_markup_buf(buf, "<OutputIntent>%s</OutputIntent>\n",
+		conf_get_name(intentNames, c->intent[out_profile]));
+    if (c->intent[display_profile]!=conf_default.intent[display_profile])
+        buf = uf_markup_buf(buf, "<DisplayIntent>%s</DisplayIntent>\n",
+		conf_get_name(intentNames, c->intent[display_profile]));
     /* We always write the Make and Mode information
      * to know if the WB setting is relevant */
     buf = uf_markup_buf(buf, "<Make>%s</Make>\n", c->make);
@@ -848,6 +921,8 @@ int conf_save(conf_data *c, char *IDFilename, char **confBuffer)
 	    buf = uf_markup_buf(buf,
 		    "<DarkframeFile>%s</DarkframeFile>\n", c->darkframeFile);
 	buf = uf_markup_buf(buf, "<Timestamp>%s</Timestamp>\n", c->timestamp);
+	buf = uf_markup_buf(buf, "<Orientation>%d</Orientation>\n",
+		c->orientation);
 	buf = uf_markup_buf(buf, "<ISOSpeed>%s</ISOSpeed>\n", c->isoText);
 	buf = uf_markup_buf(buf, "<Shutter>%s</Shutter>\n", c->shutterText);
 	buf = uf_markup_buf(buf, "<Aperture>%s</Aperture>\n", c->apertureText);
@@ -859,6 +934,8 @@ int conf_save(conf_data *c, char *IDFilename, char **confBuffer)
 	    buf = uf_markup_buf(buf, "<Lens>%s</Lens>\n", c->lensText);
 	buf = uf_markup_buf(buf, "<EXIFSource>%s</EXIFSource>\n",
 		c->exifSource);
+	buf = uf_markup_buf(buf, "<Crop>%d %d %d %d</Crop>\n",
+		c->CropX1, c->CropY1, c->CropX2, c->CropY2);
 	char *log = ufraw_message(UFRAW_GET_LOG, NULL);
 	if (log!=NULL) {
 	    char *utf8 = g_filename_to_utf8(log, -1, NULL, NULL, NULL);
@@ -911,6 +988,12 @@ void conf_copy_image(conf_data *dst, const conf_data *src)
      * since on different make and model ChanMul are meaningless */
     g_strlcpy(dst->make, src->make, max_name);
     g_strlcpy(dst->model, src->model, max_name);
+    dst->orientation = src->orientation;
+    dst->CropX1 = src->CropX1;
+    dst->CropY1 = src->CropY1;
+    dst->CropX2 = src->CropX2;
+    dst->CropY2 = src->CropY2;
+    dst->threshold = src->threshold;
     dst->exposure = src->exposure;
     dst->ExposureNorm = src->ExposureNorm;
     dst->saturation = src->saturation;
@@ -970,8 +1053,8 @@ void conf_copy_image(conf_data *dst, const conf_data *src)
 	    dst->curveCount++;
 	}
     }
-    /* We only copy the current input/output profile */
-    for (j=0; j<2; j++) {
+    /* We only copy the current input/output/display profile */
+    for (j=0; j<profile_types; j++) {
 	if (src->profileIndex[j]==0) {
 	    dst->profileIndex[j] = src->profileIndex[j];
 	    dst->profile[j][0] = src->profile[j][0];
@@ -997,7 +1080,8 @@ void conf_copy_image(conf_data *dst, const conf_data *src)
 	    }
 	}
     }
-    dst->intent = src->intent;
+    dst->intent[out_profile] = src->intent[out_profile];
+    dst->intent[display_profile] = src->intent[display_profile];
 }
 
 /* Copy the 'save options' from *src to *dst */
@@ -1037,6 +1121,7 @@ int conf_set_cmd(conf_data *conf, const conf_data *cmd)
     if (cmd->autoExposure) {
 	conf->autoExposure = cmd->autoExposure;
     }
+    if (cmd->threshold!=NULLF) conf->threshold = cmd->threshold;
     if (cmd->exposure!=NULLF) {
 	conf->exposure = cmd->exposure;
 	conf->autoExposure = disabled_state;
@@ -1149,11 +1234,13 @@ N_("--clip=digital|film   Clip highlights for positive EV.\n"
 N_("--gamma=GAMMA         Gamma adjustment of the base curve (default 0.45).\n"),
 N_("--linearity=LINEARITY Linearity of the base curve (default 0.10).\n"),
 N_("--saturation=SAT      Saturation adjustment (default 1.0, 0 for B&W output).\n"),
+N_("--wavelet-denoising-threshold=THRESHOLD\n"
+"                      Wavelet denoising threshold (default 0.0).\n"),
 N_("--exposure=auto|EXPOSURE\n"
 "                      Auto exposure or exposure correction in EV (default 0).\n"),
 N_("--black-point=auto|BLACK\n"
 "                      Auto black-point or black-point value (default 0).\n"),
-N_("--interpolation=ahd|vng|four-color|bilinear\n"
+N_("--interpolation=eahd|ahd|vng|four-color|ppg|bilinear\n"
 "                      Interpolation algorithm to use (default ahd).\n"),
 "\n",
 N_("The options which are related to the final output are:\n"),
@@ -1186,44 +1273,37 @@ N_("Last, but not least, --version displays the version number and compilation\n
 "END"
 };
 
-char versionText[]= 
-"%s " VERSION "\n" 
- 
-"EXIF " 
-#ifdef HAVE_LIBEXIF 
-"enabled.\n" 
-#else 
-"disabled.\n" 
-#endif 
- 
-"EXIV2 " 
-#ifdef HAVE_EXIV2 
-"enabled.\n" 
-#else 
-"disabled.\n" 
-#endif 
- 
-"JPEG "  
-#ifdef HAVE_LIBJPEG 
-"enabled.\n" 
-#else 
-"disabled.\n" 
-#endif 
- 
-"TIFF "  
-#ifdef HAVE_LIBTIFF 
-"enabled.\n" 
-#else 
-"disabled.\n" 
-#endif 
- 
-"ZIP "  
-#ifdef HAVE_LIBZ 
-"enabled.\n" 
-#else 
-"disabled.\n" 
-#endif 
-""; 
+char versionText[]=
+"%s " VERSION "\n"
+
+"EXIV2 "
+#ifdef HAVE_EXIV2
+EXV_PACKAGE_VERSION "\n"
+#else
+"disabled.\n"
+#endif
+
+"JPEG "
+#ifdef HAVE_LIBJPEG
+"enabled.\n"
+#else
+"disabled.\n"
+#endif
+
+"TIFF "
+#ifdef HAVE_LIBTIFF
+"enabled.\n"
+#else
+"disabled.\n"
+#endif
+
+"ZIP "
+#ifdef HAVE_LIBZ
+"enabled.\n"
+#else
+"disabled.\n"
+#endif
+"";
 
 /* ufraw_process_args returns values:
  * -1     : an error occurred.
@@ -1251,6 +1331,7 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
         {"gamma", 1, 0, 'G'},
         {"linearity", 1, 0, 'L'},
         {"saturation", 1, 0, 's'},
+        {"wavelet-denoising-threshold", 1, 0, 'n'},
         {"exposure", 1, 0, 'e'},
         {"black-point", 1, 0, 'k'},
         {"interpolation", 1, 0, 'i'},
@@ -1262,8 +1343,8 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
         {"out-path", 1, 0, 'p'},
         {"output", 1, 0, 'o'},
         {"darkframe",1,0,'a'},
-        {"restore", 0, 0, 'r'},
-        {"clip", 0, 0, 'u'},
+        {"restore", 1, 0, 'r'},
+        {"clip", 1, 0, 'u'},
         {"conf", 1, 0, 'C'},
 /* Binary flags that don't have a value are here at the end */
         {"zip", 0, 0, 'z'},
@@ -1281,7 +1362,7 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
     void *optPointer[] = { &wbName, &cmd->temperature, &cmd->green,
 	&baseCurveName, &baseCurveFile, &curveName, &curveFile,
         &cmd->profile[0][0].gamma, &cmd->profile[0][0].linear,
-	&cmd->saturation,
+	&cmd->saturation, &cmd->threshold, 
         &cmd->exposure, &cmd->black, &interpolationName,
 	&cmd->shrink, &cmd->size, &cmd->compression,
 	&outTypeName, &createIDName, &outPath, &output, &darkframeFile,
@@ -1297,6 +1378,7 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
     cmd->profile[0][0].linear=NULLF;
     cmd->saturation=NULLF;
     cmd->black=NULLF;
+    cmd->threshold=NULLF;
     cmd->exposure=NULLF;
     cmd->temperature=NULLF;
     cmd->green=NULLF;
@@ -1323,6 +1405,7 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
         case 'G':
         case 'L':
         case 's':
+	case 'n':
         case 'd':
             if (sscanf(optarg, "%lf", (double *)optPointer[index])==0) {
                 ufraw_message(UFRAW_ERROR,
@@ -1623,7 +1706,7 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
 	cmd->darkframe = ufraw_load_darkframe(df);
         g_strlcpy(cmd->darkframeFile, df, max_path);
 	g_free(df);
-    }    
+    }
     /* cmd->inputFilename is used to store the conf file */
     g_strlcpy(cmd->inputFilename, "", max_path);
     if (conf!=NULL)
