@@ -21,6 +21,17 @@
 #include "nikon_curve.h"
 #include "ufraw.h"
 
+int lcms_message(int ErrorCode, const char *ErrorText)
+{
+    /* Possible ErrorCode:
+     * LCMS_ERRC_WARNING        0x1000
+     * LCMS_ERRC_RECOVERABLE    0x2000
+     * LCMS_ERRC_ABORTED        0x3000 */
+    (void)ErrorCode;
+    ufraw_message(UFRAW_ERROR, "%s", ErrorText);
+    return 1; /* Tell lcms that we handled the error */
+}
+
 developer_data *developer_init()
 {
     int i;
@@ -41,10 +52,11 @@ developer_data *developer_init()
     TransferFunction[0] = cmsAllocGamma(0x100);
     TransferFunction[1] = TransferFunction[2] = cmsBuildGamma(0x100, 1.0);
     d->saturationProfile = NULL;
-    d->intent = -1;
+    d->intent[out_profile] = -1;
+    d->intent[display_profile] = -1;
     d->updateTransform = TRUE;
     d->colorTransform = NULL;
-    cmsSetErrorHandler((void *)ufraw_message);
+    cmsSetErrorHandler(lcms_message);
     return d;
 }
 
@@ -89,20 +101,27 @@ void developer_profile(developer_data *d, int type, profile_data *p)
     }
 }
 
-int saturation_sampler(register WORD In[], register WORD Out[], register LPVOID Cargo){
-    cmsCIELab LabIn, LabOut;
-    cmsCIELCh LChIn, LChOut;
-    double saturation = *(double *)Cargo;
-    cmsLabEncoded2Float(&LabIn, In);
-    cmsLab2LCh(&LChIn, &LabIn);
-    // Do some adjusts on LCh
-    LChOut.L = LChIn.L;
-    LChOut.C = (1-pow(1-(LChIn.C/256.), saturation))*256;
-    LChOut.h = LChIn.h;
+#define CLAMP_AB_DOUBLE(ab) \
+	( (ab<-128.0) ? -128.0 : ( (ab>127.9961) ? +127.9961 : ab) )
 
-    cmsLCh2Lab(&LabOut, &LChOut);
-    // Back to encoded
-    cmsFloat2LabEncoded(Out, &LabOut);
+int saturation_sampler(register WORD In[], register WORD Out[],
+	register LPVOID Cargo)
+{
+    cmsCIELab Lab;
+    double saturation = *(double *)Cargo;
+
+    cmsLabEncoded2Float(&Lab, In);
+
+    if ( Lab.a!=0.0 || Lab.b!=0.0 ) {
+
+	/* Normalized Chroma of current color (0.0 to 1.0) */
+	double Cn = MAX( fabs(Lab.a), fabs(Lab.b) ) / 128.0;
+
+	double scale = ( 1.0 - pow( 1.0 - Cn , saturation ) ) / Cn;
+	Lab.a = CLAMP_AB_DOUBLE(Lab.a * scale);
+	Lab.b = CLAMP_AB_DOUBLE(Lab.b * scale);
+    }
+    cmsFloat2LabEncoded(Out, &Lab);
 
     return TRUE;
 }
@@ -154,19 +173,106 @@ double findExpCoeff(double b)
     }
     return a;
 }
+
+void developer_create_transform(developer_data *d, DeveloperMode mode)
+{
+    if ( mode==auto_developer ) {
+	/* For auto-tools we ignore all the output settings:
+	 * luminosity, saturation, output profile and proofing.
+	 * Instead we create an sRGB output. */
+	cmsHPROFILE *sRGBprofile = cmsCreate_sRGBProfile();
+	d->colorTransform = cmsCreateTransform(
+		d->profile[in_profile], TYPE_RGB_16,
+		sRGBprofile, TYPE_RGB_16, d->intent[out_profile], 0);
+        cmsCloseProfile(sRGBprofile);
+	return;
+    }
+    int targetProfile;
+    if ( mode==file_developer ) {
+	targetProfile = out_profile;
+    } else { /* mode==display_developer */
+	targetProfile = display_profile;
+    }
+    /* When softproofing is disabled, use the out_profile intent. */
+    if ( mode==file_developer ||
+	 d->intent[display_profile]==disable_intent ) {
+	/* No need for proofing transformation. */
+	if ( strcmp(d->profileFile[in_profile],"")==0 && 
+	     strcmp(d->profileFile[targetProfile],"")==0 &&
+	     d->luminosityProfile==NULL && d->saturationProfile==NULL ) {
+	    /* No transformation at all. */
+            d->colorTransform = NULL;
+#if defined(LCMS_VERSION) && LCMS_VERSION <= 113 /* Bypass a lcms 1.13 bug. */
+	} else if ( d->luminosityProfile==NULL && d->saturationProfile==NULL ) {
+	    d->colorTransform = cmsCreateTransform(
+		    d->profile[in_profile], TYPE_RGB_16,
+		    d->profile[targetProfile], TYPE_RGB_16,
+		    d->intent[out_profile], 0);
+#endif
+        } else {
+	    cmsHPROFILE prof[4];
+	    int i = 0;
+	    prof[i++] = d->profile[in_profile];
+	    if ( d->luminosityProfile!=NULL )
+		prof[i++] = d->luminosityProfile;
+	    if ( d->saturationProfile!=NULL )
+		prof[i++] = d->saturationProfile;
+	    prof[i++] = d->profile[targetProfile];
+	    d->colorTransform = cmsCreateMultiprofileTransform(prof, i,
+		    TYPE_RGB_16, TYPE_RGB_16, d->intent[out_profile], 0);
+	}
+    } else {
+	/* Create a proofing profile */
+	if ( d->luminosityProfile==NULL && d->saturationProfile==NULL ) {
+	    /* No intermediate profiles, we can use lcms proofing directly. */
+	    d->colorTransform = cmsCreateProofingTransform(
+		    d->profile[in_profile], TYPE_RGB_16,
+		    d->profile[display_profile], TYPE_RGB_16,
+		    d->profile[out_profile],
+		    d->intent[display_profile], d->intent[out_profile],
+		    cmsFLAGS_SOFTPROOFING);
+	} else {
+	    /* Following code imitates the function
+	     * cmsCreateMultiprofileProofingTransform(),
+	     * which does not exist in lcms. */
+	    cmsHPROFILE prof[3];
+	    int i = 0;
+	    prof[i++] = d->profile[in_profile];
+	    if ( d->luminosityProfile!=NULL )
+		prof[i++] = d->luminosityProfile;
+	    if ( d->saturationProfile!=NULL )
+		prof[i++] = d->saturationProfile;
+	    d->colorTransform = cmsCreateMultiprofileTransform(prof, i,
+		    TYPE_RGB_16, NOCOLORSPACECHECK(TYPE_RGB_16),
+		    d->intent[display_profile], cmsFLAGS_SOFTPROOFING);
+
+	    prof[0] = cmsTransform2DeviceLink(d->colorTransform,
+		    cmsFLAGS_GUESSDEVICECLASS);
+	    cmsDeleteTransform(d->colorTransform);
+	    d->colorTransform = cmsCreateProofingTransform(
+		    prof[0], TYPE_RGB_16,
+		    d->profile[display_profile], TYPE_RGB_16,
+		    d->profile[out_profile],
+		    d->intent[display_profile], d->intent[out_profile],
+		    cmsFLAGS_SOFTPROOFING);
+	}
+    }
+}
+
 void developer_prepare(developer_data *d, conf_data *conf,
     int rgbMax, float rgb_cam[3][4], int colors, int useMatrix,
-    gboolean autoTools)
+    DeveloperMode mode)
 {
     unsigned c, i;
-    profile_data *in, *out;
+    profile_data *in, *out, *display;
     CurveData *baseCurve, *curve;
 
     in = &conf->profile[in_profile][conf->profileIndex[in_profile]];
     out = &conf->profile[out_profile][conf->profileIndex[out_profile]];
+    display = &conf->profile[display_profile]
+		[conf->profileIndex[display_profile]];
     baseCurve = &conf->BaseCurve[conf->BaseCurveIndex];
     curve = &conf->curve[conf->curveIndex];
-    if (autoTools) curve = NULL;
 
     d->rgbMax = rgbMax;
     d->colors = colors;
@@ -247,86 +353,60 @@ void developer_prepare(developer_data *d, conf_data *conf,
 		d->gammaCurve[i] = MIN(pow(a*BaseCurve[FilmCurve[i]]/0x10000+b,
 					   g)*0x10000, 0xFFFF);
     }
-    developer_profile(d, 0, in);
-    developer_profile(d, 1, out);
-    if ( conf->intent!=d->intent ) {
-        d->intent = conf->intent;
+    developer_profile(d, in_profile, in);
+    developer_profile(d, out_profile, out);
+    developer_profile(d, display_profile, display);
+    if ( conf->intent[out_profile]!=d->intent[out_profile] ) {
+        d->intent[out_profile] = conf->intent[out_profile];
         d->updateTransform = TRUE;
     }
-    if (curve==NULL) {
-	/* Reset the luminosity profile and make sure it gets recalculated */
-	cmsCloseProfile(d->luminosityProfile);
-	d->luminosityProfile = NULL;
-	d->luminosityCurveData.m_gamma = -1.0;
+    if ( conf->intent[display_profile]!=d->intent[display_profile] ) {
+        d->intent[display_profile] = conf->intent[display_profile];
         d->updateTransform = TRUE;
-    } else {
-	/* Check if curve data has changed. */
-	if (memcmp(curve, &d->luminosityCurveData, sizeof(CurveData))) {
-	    d->luminosityCurveData = *curve;
-	    /* Trivial curve does not require a profile */
-	    if ( CurveDataIsTrivial(curve) ) {
+    }
+    /* Check if curve data has changed. */
+    if (memcmp(curve, &d->luminosityCurveData, sizeof(CurveData))) {
+	d->luminosityCurveData = *curve;
+	/* Trivial curve does not require a profile */
+	if ( CurveDataIsTrivial(curve) ) {
+	    d->luminosityProfile = NULL;
+	} else {
+	    cmsCloseProfile(d->luminosityProfile);
+	    CurveSample *cs = CurveSampleInit(0x100, 0x10000);
+	    ufraw_message(UFRAW_RESET, NULL);
+	    if (CurveDataSample(curve, cs)!=UFRAW_SUCCESS) {
+		ufraw_message(UFRAW_REPORT, NULL);
 		d->luminosityProfile = NULL;
 	    } else {
-		cmsCloseProfile(d->luminosityProfile);
-		CurveSample *cs = CurveSampleInit(0x100, 0x10000);
-		ufraw_message(UFRAW_RESET, NULL);
-		if (CurveDataSample(curve, cs)!=UFRAW_SUCCESS) {
-		    ufraw_message(UFRAW_REPORT, NULL);
-		    d->luminosityProfile = NULL;
-		} else {
-		    LPGAMMATABLE *TransferFunction =
+		LPGAMMATABLE *TransferFunction =
 			(LPGAMMATABLE *)d->TransferFunction;
-		    for (i=0; i<0x100; i++)
-			TransferFunction[0]->GammaTable[i] = cs->m_Samples[i];
-		    d->luminosityProfile = cmsCreateLinearizationDeviceLink(
-			    icSigLabData, TransferFunction);
-		    cmsSetDeviceClass(d->luminosityProfile, icSigAbstractClass);
-		}
-		CurveSampleFree(cs);
+		for (i=0; i<0x100; i++)
+		    TransferFunction[0]->GammaTable[i] = cs->m_Samples[i];
+		d->luminosityProfile = cmsCreateLinearizationDeviceLink(
+			icSigLabData, TransferFunction);
+		cmsSetDeviceClass(d->luminosityProfile, icSigAbstractClass);
 	    }
-	    d->updateTransform = TRUE;
+	    CurveSampleFree(cs);
 	}
+	d->updateTransform = TRUE;
     }
     if ( conf->saturation!=d->saturation ) {
-        d->saturation = conf->saturation;
+	d->saturation = conf->saturation;
 	cmsCloseProfile(d->saturationProfile);
 	if (d->saturation==1)
 	    d->saturationProfile = NULL;
 	else
 	    d->saturationProfile = create_saturation_profile(d->saturation);
-        d->updateTransform = TRUE;
+	d->updateTransform = TRUE;
     }
-    if (d->updateTransform) {
+    if (d->updateTransform || mode==auto_developer) {
         if (d->colorTransform!=NULL)
             cmsDeleteTransform(d->colorTransform);
-        if (!strcmp(d->profileFile[0],"") && !strcmp(d->profileFile[1],"") &&
-	    d->luminosityProfile==NULL && d->saturationProfile==NULL) {
-            d->colorTransform = NULL;
-#if defined(LCMS_VERSION) && LCMS_VERSION <= 113 /* Bypass a lcms 1.13 bug. */
-        } else if (d->luminosityProfile==NULL && d->saturationProfile==NULL) {
-	    d->colorTransform = cmsCreateTransform(d->profile[0],
-		    TYPE_RGB_16, d->profile[1], TYPE_RGB_16, d->intent, 0);
-#endif
-        } else {
-	    cmsHPROFILE prof[4];
-	    i = 0;
-	    prof[i++] = d->profile[0];
-	    if (d->luminosityProfile!=NULL)
-		prof[i++] = d->luminosityProfile;
-	    if (d->saturationProfile!=NULL)
-		prof[i++] = d->saturationProfile;
-	    prof[i++] = d->profile[1];
-	    d->colorTransform = cmsCreateMultiprofileTransform(prof, i,
-		    TYPE_RGB_16, TYPE_RGB_16, d->intent, 0);
-	}
+	developer_create_transform(d, mode);
     }
 }
 
 extern const double xyz_rgb[3][3];
-//const double xyz_rgb[3][3] = {                  /* XYZ from RGB */
-//  { 0.412453, 0.357580, 0.180423 },
-//  { 0.212671, 0.715160, 0.072169 },
-//  { 0.019334, 0.119193, 0.950227 } };
 const double rgb_xyz[3][3] = {			/* RGB from XYZ */
     { 3.24048, -1.53715, -0.498536 },
     { -0.969255, 1.87599, 0.0415559 },
@@ -410,116 +490,12 @@ inline void develope(void *po, guint16 pix[4], developer_data *d, int mode,
     guint16 *buf, int count)
 {
     guint8 *p8 = po;
-    guint16 *p16 = po, c, cc;
-    gint64 tmp, tmppix[4];
+    guint16 *p16 = po, c, tmppix[3];
     int i;
-    gboolean clipped;
     for (i=0; i<count; i++) {
-	clipped = FALSE;
-	for (c=0; c<d->colors; c++) {
-	    /* Set WB, normalizing tmppix[c]<0x10000 */
-	    tmppix[c] = (guint64)pix[i*4+c] * d->rgbWB[c] / 0x10000;
-	    if ( d->restoreDetails!=clip_details &&
-		 tmppix[c] > d->max ) {
-		clipped = TRUE;
-	    } else {
-		tmppix[c] = MIN(tmppix[c], d->max);
-	    }
-	    /* We are counting on the fact that film_highlights
-	     * and !clip_highlights cannot be set simultaneously. */
-	    if ( d->clipHighlights==film_highlights )
-		tmppix[c] = tmppix[c] * 0x10000 / d->max;
-	    else
-		tmppix[c] = tmppix[c] * d->exposure / d->max;
-	}
-	if ( clipped ) {
-	    /* At this point a value of d->exposure in tmppix[c] corresponds
-	     * to "1.0" (full exposure). Still the maximal value can be
-	     * d->exposure * 0x10000 / d->max */
-	    gint64 unclippedPix[3], clippedPix[3];
-	    if ( d->useMatrix ) {
-		for (cc=0; cc<3; cc++) {
-		    for (c=0, tmp=0; c<d->colors; c++)
-			tmp += tmppix[c] * d->colorMatrix[cc][c];
-		    unclippedPix[cc] = MAX(tmp / 0x10000, 0);
-		}
-	    } else {
-		for (c=0; c<3; c++) unclippedPix[c] = tmppix[c];
-	    }
-	    for (c=0; c<3; c++) tmppix[c] = MIN(tmppix[c], d->exposure);
-	    if ( d->useMatrix ) {
-		for (cc=0; cc<3; cc++) {
-		    for (c=0, tmp=0; c<d->colors; c++)
-			tmp += tmppix[c] * d->colorMatrix[cc][c];
-		    clippedPix[cc] = MAX(tmp/0x10000, 0);
-		}
-	    } else {
-		for (c=0; c<3; c++) clippedPix[c] = tmppix[c];
-	    }
-	    if ( d->restoreDetails==restore_lch_details ) {
-		float lch[3], clippedLch[3], unclippedLch[3];
-		rgb_to_cielch(unclippedPix, unclippedLch);
-		rgb_to_cielch(clippedPix, clippedLch);
-		//lch[0] = clippedLch[0] + (unclippedLch[0]-clippedLch[0]) * x;
-		lch[0] = unclippedLch[0];
-		lch[1] = clippedLch[1];
-		lch[2] = clippedLch[2];
-		cielch_to_rgb(lch, tmppix);
-	    } else { /* restore_hsv_details */
-		int maxc, midc, minc;
-		MaxMidMin(unclippedPix, &maxc, &midc, &minc);
-		gint64 unclippedLum = unclippedPix[maxc];
-		gint64 clippedLum = clippedPix[maxc];
-		/*gint64 unclippedSat;
-		if ( unclippedPix[maxc]==0 )
-		    unclippedSat = 0;
-		else
-		    unclippedSat = 0x10000 -
-			    unclippedPix[minc] * 0x10000 / unclippedPix[maxc];*/
-		gint64 clippedSat;
-		if ( clippedPix[maxc]<clippedPix[minc] || clippedPix[maxc]==0 )
-		    clippedSat = 0;
-		else
-		    clippedSat = 0x10000 -
-			    clippedPix[minc] * 0x10000 / clippedPix[maxc];
-		gint64 clippedHue;
-		if ( clippedPix[maxc]==clippedPix[minc] ) clippedHue = 0;
-		else clippedHue =
-			(clippedPix[midc]-clippedPix[minc])*0x10000 /
-			(clippedPix[maxc]-clippedPix[minc]);
-		gint64 unclippedHue;
-		if ( unclippedPix[maxc]==unclippedPix[minc] )
-		    unclippedHue = clippedHue;
-		else
-		    unclippedHue =
-			(unclippedPix[midc]-unclippedPix[minc])*0x10000 /
-			(unclippedPix[maxc]-unclippedPix[minc]);
-		/* Here we decide how to mix the clipped and unclipped values.
-		 * The general equation is clipped + (unclipped - clipped) * x,
-		 * where x is between 0 and 1. */
-		/* For lum we set x=1/2. Thus hightlights are not too bright. */
-		gint64 lum = clippedLum + (unclippedLum - clippedLum) * 1/2;
-		/* For sat we should set x=0 to prevent color artifacts. */
-		//gint64 sat = clippedSat + (unclippedSat - clippedSat) * 0/1 ;
-		gint64 sat = clippedSat;
-		/* For hue we set x=1. This doesn't seem to have much effect. */
-		gint64 hue = unclippedHue;
-
-		tmppix[maxc] = lum;
-		tmppix[minc] = lum * (0x10000-sat) / 0x10000;
-		tmppix[midc] = lum * (0x10000-sat + sat*hue/0x10000) / 0x10000;
-	    }
-	} else { /* !clipped */
-	    if (d->useMatrix) {
-		for (cc=0; cc<3; cc++) {
-		    for (c=0, tmp=0; c<d->colors; c++)
-			tmp += tmppix[c] * d->colorMatrix[cc][c];
-		    tmppix[cc] = tmp/0x10000;
-		}
-	    }
-	}
+	develop_linear(pix+i*4, tmppix, d);
 	for (c=0; c<3; c++)
-	    buf[i*3+c] = d->gammaCurve[MIN(MAX(tmppix[c], 0), 0xFFFF)];
+	    buf[i*3+c] = d->gammaCurve[tmppix[c]];
     }
     if (d->colorTransform!=NULL)
         cmsDoTransform(d->colorTransform, buf, buf, count);
@@ -527,3 +503,123 @@ inline void develope(void *po, guint16 pix[4], developer_data *d, int mode,
     if (mode==16) for (i=0; i<3*count; i++) p16[i] = buf[i];
     else for (i=0; i<3*count; i++) p8[i] = buf[i] >> 8;
 }
+
+void develop_linear(guint16 in[4], guint16 out[3], developer_data *d)
+{
+    unsigned c, cc;
+    gint64 tmppix[4], tmp;
+    gboolean clipped = FALSE;
+    for (c=0; c<d->colors; c++) {
+	/* Set WB, normalizing tmppix[c]<0x10000 */
+	tmppix[c] = (guint64)in[c] * d->rgbWB[c] / 0x10000;
+	if ( d->restoreDetails!=clip_details &&
+	    tmppix[c] > d->max ) {
+	    clipped = TRUE;
+	} else {
+	    tmppix[c] = MIN(tmppix[c], d->max);
+	}
+	/* We are counting on the fact that film_highlights
+	 * and !clip_highlights cannot be set simultaneously. */
+	if ( d->clipHighlights==film_highlights )
+	    tmppix[c] = tmppix[c] * 0x10000 / d->max;
+	else
+	    tmppix[c] = tmppix[c] * d->exposure / d->max;
+    }
+    if ( clipped ) {
+	/* At this point a value of d->exposure in tmppix[c] corresponds
+	 * to "1.0" (full exposure). Still the maximal value can be
+	 * d->exposure * 0x10000 / d->max */
+	gint64 unclippedPix[3], clippedPix[3];
+	if ( d->useMatrix ) {
+	    for (cc=0; cc<3; cc++) {
+		for (c=0, tmp=0; c<d->colors; c++)
+		    tmp += tmppix[c] * d->colorMatrix[cc][c];
+		unclippedPix[cc] = MAX(tmp/0x10000, 0);
+	    }
+	} else {
+	    for (c=0; c<3; c++) unclippedPix[c] = tmppix[c];
+	}
+	for (c=0; c<3; c++) tmppix[c] = MIN(tmppix[c], d->exposure);
+	if ( d->useMatrix ) {
+	    for (cc=0; cc<3; cc++) {
+		for (c=0, tmp=0; c<d->colors; c++)
+		    tmp += tmppix[c] * d->colorMatrix[cc][c];
+		clippedPix[cc] = MAX(tmp/0x10000, 0);
+	    }
+	} else {
+	    for (c=0; c<3; c++) clippedPix[c] = tmppix[c];
+	}
+	if ( d->restoreDetails==restore_lch_details ) {
+	    float lch[3], clippedLch[3], unclippedLch[3];
+	    rgb_to_cielch(unclippedPix, unclippedLch);
+	    rgb_to_cielch(clippedPix, clippedLch);
+	    //lch[0] = clippedLch[0] + (unclippedLch[0]-clippedLch[0]) * x;
+	    lch[0] = unclippedLch[0];
+	    lch[1] = clippedLch[1];
+	    lch[2] = clippedLch[2];
+	    cielch_to_rgb(lch, tmppix);
+	} else { /* restore_hsv_details */
+	    int maxc, midc, minc;
+	    MaxMidMin(unclippedPix, &maxc, &midc, &minc);
+	    gint64 unclippedLum = unclippedPix[maxc];
+	    gint64 clippedLum = clippedPix[maxc];
+	    /*gint64 unclippedSat;
+	    if ( unclippedPix[maxc]==0 )
+	        unclippedSat = 0;
+	    else
+	        unclippedSat = 0x10000 -
+			unclippedPix[minc] * 0x10000 / unclippedPix[maxc];*/
+	    gint64 clippedSat;
+	    if ( clippedPix[maxc]<clippedPix[minc] || clippedPix[maxc]==0 )
+		clippedSat = 0;
+	    else
+		clippedSat = 0x10000 -
+			clippedPix[minc] * 0x10000 / clippedPix[maxc];
+	    gint64 clippedHue;
+	    if ( clippedPix[maxc]==clippedPix[minc] ) clippedHue = 0;
+	    else clippedHue =
+		    (clippedPix[midc]-clippedPix[minc])*0x10000 /
+		    (clippedPix[maxc]-clippedPix[minc]);
+	    gint64 unclippedHue;
+	    if ( unclippedPix[maxc]==unclippedPix[minc] )
+		unclippedHue = clippedHue;
+	    else
+		unclippedHue =
+		    (unclippedPix[midc]-unclippedPix[minc])*0x10000 /
+		    (unclippedPix[maxc]-unclippedPix[minc]);
+	    /* Here we decide how to mix the clipped and unclipped values.
+	     * The general equation is clipped + (unclipped - clipped) * x,
+	     * where x is between 0 and 1. */
+	    /* For lum we set x=1/2. Thus hightlights are not too bright. */
+	    gint64 lum = clippedLum + (unclippedLum - clippedLum) * 1/2;
+	    /* For sat we should set x=0 to prevent color artifacts. */
+	    //gint64 sat = clippedSat + (unclippedSat - clippedSat) * 0/1 ;
+	    gint64 sat = clippedSat;
+	    /* For hue we set x=1. This doesn't seem to have much effect. */
+	    gint64 hue = unclippedHue;
+
+	    tmppix[maxc] = lum;
+	    tmppix[minc] = lum * (0x10000-sat) / 0x10000;
+	    tmppix[midc] = lum * (0x10000-sat + sat*hue/0x10000) / 0x10000;
+	}
+    } else { /* !clipped */
+	if (d->useMatrix) {
+	    gint64 tmp[3];
+	    for (cc=0; cc<3; cc++) {
+		for (c=0, tmp[cc]=0; c<d->colors; c++)
+		    tmp[cc] += tmppix[c] * d->colorMatrix[cc][c];
+	    }
+	    for (c=0; c<3; c++) tmppix[c] = MAX(tmp[c]/0x10000, 0);
+	}
+	gint64 max = tmppix[0];
+	for (c=1; c<3; c++) max = MAX(tmppix[c], max);
+	if (max > 0xFFFF) {
+	    gint64 unclippedLum = max;
+	    gint64 clippedLum = 0xFFFF;
+	    gint64 lum = clippedLum + (unclippedLum - clippedLum) * 1/4;
+	    for (c=0; c<3; c++) tmppix[c] = tmppix[c] * lum / max;
+	}
+    }
+    for (c=0; c<3; c++)
+	out[c] = MIN(MAX(tmppix[c], 0), 0xFFFF);
+}
