@@ -21,6 +21,12 @@
 #include <math.h>
 #include <time.h>
 #include <errno.h>
+#ifdef HAVE_LIBZ
+#include <zlib.h>
+#endif
+#ifdef HAVE_LIBBZ2
+#include <bzlib.h>
+#endif
 #include <glib.h>
 #include <glib/gi18n.h>
 #include "dcraw_api.h"
@@ -45,10 +51,8 @@ void ufraw_batch_messenger(char *message)
      * (not including possibly one at the end).
      * Otherwise, the header will be printed only for the first line. */
     if (g_strstr_len(message, strlen(message)-1, "\n")==NULL)
-	fprintf(stderr, "%s: ", ufraw_binary);
-    fprintf(stderr, "%s", message);
-    if (message[strlen(message)-1]!='\n') fputc('\n', stderr);
-    fflush(stderr);
+	g_printerr("%s: ", ufraw_binary);
+    g_printerr("%s%c", message, message[strlen(message)-1]!='\n' ? '\n' : 0);
 }
 
 char *ufraw_message(int code, const char *format, ...)
@@ -105,12 +109,6 @@ char *ufraw_message(int code, const char *format, ...)
 		    ufraw_messenger(message, parentWindow);
                 g_free(message);
                 return NULL;
-    case UFRAW_LCMS_WARNING:
-    case UFRAW_LCMS_RECOVERABLE:
-    case UFRAW_LCMS_ABORTED:
-                ufraw_messenger(message, parentWindow);
-                g_free(message);
-                return (char *)1; /* Tell lcms that we are handling the error */
     case UFRAW_REPORT:
                 ufraw_messenger(errorBuffer, parentWindow);
                 return NULL;
@@ -121,6 +119,97 @@ char *ufraw_message(int code, const char *format, ...)
     }
 }
 
+static int make_temporary(char *basefilename, char **tmpfilename)
+{
+    int fd;
+    char *basename = g_path_get_basename(basefilename);
+    char *template = g_strconcat(basename, ".tmp.XXXXXX", NULL);
+    fd = g_file_open_tmp(template, tmpfilename, NULL);
+    g_free(template);
+    g_free(basename);
+    return fd;
+}
+
+static ssize_t writeall(int fd, const char *buf, ssize_t size)
+{
+  ssize_t written;
+  ssize_t wr;
+  for (written = 0; size > 0; size -= wr, written += wr, buf += wr)
+	if ((wr = write(fd, buf, size)) < 0)
+	    break;
+  return written;
+}
+
+static char *decompress_gz(char *origfilename)
+{
+#ifdef HAVE_LIBZ
+    char *filename;
+    int tmpfd;
+    gzFile gzfile;
+    char buf[8192];
+    ssize_t size;
+    if ((tmpfd = make_temporary(origfilename, &filename)) != -1) {
+	if ((gzfile = gzopen(origfilename, "rb")) != 0) {
+	    while ((size = gzread(gzfile, buf, sizeof buf)) > 0) {
+		if (writeall(tmpfd, buf, size) != size)
+		    break;
+	    }
+	    gzclose(gzfile);
+	    if (size == 0)
+		if (close(tmpfd) == 0)
+		    return filename;
+	}
+	close(tmpfd);
+	unlink(filename);
+	free(filename);
+    }
+#else
+    origfilename = origfilename;
+    ufraw_message(UFRAW_SET_ERROR,
+		  "Cannot open gzip compressed images.\n");
+#endif
+    return NULL;
+}
+
+static char *decompress_bz2(char *origfilename)
+{
+#ifdef HAVE_LIBBZ2
+    char *filename;
+    int tmpfd;
+    FILE *compfile;
+    BZFILE *bzfile;
+    int bzerror;
+    char buf[8192];
+    ssize_t size;
+
+    if ((tmpfd = make_temporary(origfilename, &filename)) != -1) {
+	if ((compfile = fopen(origfilename, "rb")) != 0) {
+	    if ((bzfile = BZ2_bzReadOpen(&bzerror, compfile,
+					 0, 0, 0, 0)) != 0) {
+	        while ((size = BZ2_bzRead(&bzerror, bzfile,
+					  buf, sizeof buf)) > 0)
+		    if (writeall(tmpfd, buf, size) != size)
+		        break;
+		BZ2_bzReadClose(&bzerror, bzfile);
+		fclose(compfile);
+		if (size == 0) {
+		    close(tmpfd);
+		    return filename;
+		}
+	    }
+	}
+	close(tmpfd);
+	unlink(filename);
+	free(filename);
+    }
+#else
+    origfilename = origfilename;
+    ufraw_message(UFRAW_SET_ERROR,
+		  "Cannot open bzip2 compressed images.\n");
+#endif
+    return NULL;
+}
+
 ufraw_data *ufraw_open(char *filename)
 {
     int status;
@@ -129,6 +218,9 @@ ufraw_data *ufraw_open(char *filename)
     ufraw_message(UFRAW_CLEAN, NULL);
     conf_data *conf = NULL;
     char *fname, *hostname;
+    char *origfilename;
+    gchar *unzippedBuf = NULL;
+    unsigned unzippedBufLen = 0;
 
     fname = g_filename_from_uri(filename, &hostname, NULL);
     if (fname!=NULL) {
@@ -177,18 +269,42 @@ ufraw_data *ufraw_open(char *filename)
 
 	filename = conf->inputFilename;
     }
+    origfilename = filename;
+    if (!strcasecmp(filename + strlen(filename) - 3, ".gz"))
+	filename = decompress_gz(filename);
+    else if (!strcasecmp(filename + strlen(filename) - 4, ".bz2"))
+	filename = decompress_bz2(filename);
+    if (filename == 0) {
+	ufraw_message(UFRAW_SET_ERROR,
+		      "Error creating temporary file for compressed data.");
+	return NULL;
+    }
     raw = g_new(dcraw_data, 1);
     status = dcraw_open(raw, filename);
+    if (filename != origfilename) {
+	g_file_get_contents(filename, &unzippedBuf, &unzippedBufLen, NULL);
+	unlink(filename);
+	free(filename);
+	filename = origfilename;
+    }
     if ( status!=DCRAW_SUCCESS) {
         /* Hold the message without displaying it */
         ufraw_message(UFRAW_SET_WARNING, raw->message);
-        g_free(raw);
-        return NULL;
+	if ( status!=DCRAW_WARNING) {
+	    g_free(raw);
+	    g_free(unzippedBuf);
+	    return NULL;
+	}
     }
     uf = g_new0(ufraw_data, 1);
+    uf->unzippedBuf = unzippedBuf;
+    uf->unzippedBufLen = unzippedBufLen;
     uf->conf = conf;
     g_strlcpy(uf->filename, filename, max_path);
     uf->image.image = NULL;
+    int i;
+    for (i=ufraw_first_phase; i<ufraw_phases_num; i++)
+	uf->Images[i].buffer = NULL;
     uf->thumb.buffer = NULL;
     uf->raw = raw;
     uf->colors = raw->colors;
@@ -196,27 +312,10 @@ ufraw_data *ufraw_open(char *filename)
     uf->developer = developer_init();
     uf->widget = NULL;
     uf->RawLumHistogram = NULL;
-    if (raw->fuji_width) {
-        /* Copied from DCRaw's fuji_rotate() */
-        uf->predictedWidth = raw->fuji_width / raw->fuji_step;
-        uf->predictedHeight = (raw->height - raw->fuji_width) / raw->fuji_step;
-    } else {
-	if (raw->pixel_aspect < 1)
-	    uf->predictedHeight = raw->height / raw->pixel_aspect + 0.5;
-	else
-	    uf->predictedHeight = raw->height;
-	if (raw->pixel_aspect > 1)
-	    uf->predictedWidth = raw->width * raw->pixel_aspect + 0.5;
-	else
-	    uf->predictedWidth = raw->width;
-    }
-    if (raw->flip & 4) {
-        int tmp = uf->predictedHeight;
-        uf->predictedHeight = uf->predictedWidth;
-        uf->predictedWidth = tmp;
-    }
+    uf->HaveFilters = raw->filters!=0;
     ufraw_message(UFRAW_SET_LOG, "ufraw_open: w:%d h:%d curvesize:%d\n",
-        uf->predictedWidth, uf->predictedHeight, raw->toneCurveSize);
+        uf->initialWidth, uf->initialHeight, raw->toneCurveSize);
+ 
     return uf;
 }
 
@@ -258,7 +357,6 @@ int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf, conf_data *cmd)
 
     /* Check if we are loading an ID file */
     if (uf!=NULL) {
-	uf->LoadingID = FALSE;
 	if (uf->conf!=NULL) {
 	    uf->LoadingID = TRUE;
 	    conf_data tmp = *rc;
@@ -268,6 +366,7 @@ int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf, conf_data *cmd)
 	    g_strlcpy(tmp.outputPath, uf->conf->outputPath, max_path);
 	    *uf->conf = tmp;
 	} else {
+	    uf->LoadingID = FALSE;
 	    uf->conf = g_new(conf_data, 1);
 	    *uf->conf = *rc;
 	}
@@ -296,6 +395,12 @@ int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf, conf_data *cmd)
 	struct stat s;
 	fstat(fileno(raw->ifp), &s);
 	g_snprintf(uf->conf->inputModTime, max_name, "%d", (int)s.st_mtime);
+
+	/*Reset crop coordinates between images.*/
+	uf->conf->CropX1 = -1;
+	uf->conf->CropY1 = -1;
+	uf->conf->CropX2 = -1;
+	uf->conf->CropY2 = -1;
     }
     /*Reset EXIF data text fields to avoid spill over between images.*/
     strcpy(uf->conf->isoText, "");
@@ -304,10 +409,12 @@ int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf, conf_data *cmd)
     strcpy(uf->conf->focalLenText, "");
     strcpy(uf->conf->focalLen35Text, "");
     strcpy(uf->conf->lensText, "");
-    if (ufraw_exif_from_raw(uf)!=UFRAW_SUCCESS) {
+    if (ufraw_exif_from_exiv2(uf)!=UFRAW_SUCCESS) {
         ufraw_message(UFRAW_SET_LOG, "Error reading EXIF data from %s\n",
                 uf->filename);
     }
+    g_free(uf->unzippedBuf);
+    uf->unzippedBuf = NULL;
     /* If we switched cameras, ignore channel multipliers and
      * change spot_wb to manual_wb */
     if ( !uf->LoadingID &&
@@ -330,7 +437,8 @@ int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf, conf_data *cmd)
 	uf->conf->timestamp[strlen(uf->conf->timestamp)-1] = '\0';
     g_strlcpy(uf->conf->make, raw->make, max_name);
     g_strlcpy(uf->conf->model, raw->model, max_name);
-    uf->conf->flip = raw->flip;
+    if ( !uf->LoadingID || uf->conf->orientation<0 )
+	uf->conf->orientation = raw->flip;
     if (uf->exifBuf==NULL) {
 	g_strlcpy(uf->conf->exifSource, "DCRaw", max_name);
 	uf->conf->iso_speed = raw->iso_speed;
@@ -410,6 +518,15 @@ int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf, conf_data *cmd)
     if (strcmp(uf->conf->darkframeFile, cmd->darkframeFile)!=0)
 	uf->conf->darkframe = ufraw_load_darkframe(uf->conf->darkframeFile);
     */
+
+    dcraw_image_dimensions(raw, uf->conf->orientation,
+	    &uf->initialHeight, &uf->initialWidth);
+    // Check crop co-ordinates.
+    if (uf->conf->CropX1 < 0) uf->conf->CropX1 = 0;
+    if (uf->conf->CropY1 < 0) uf->conf->CropY1 = 0;
+    if (uf->conf->CropX2 < 0) uf->conf->CropX2 = uf->initialWidth;
+    if (uf->conf->CropY2 < 0) uf->conf->CropY2 = uf->initialHeight;
+
     return UFRAW_SUCCESS;
 }
 
@@ -459,8 +576,9 @@ int ufraw_load_raw(ufraw_data *uf)
 	return ufraw_read_embedded(uf);
     }
     if ( (status=dcraw_load_raw(raw))!=DCRAW_SUCCESS ) {
+	ufraw_message(UFRAW_SET_LOG, raw->message);
         ufraw_message(status, raw->message);
-        return status;
+        if (status!=DCRAW_WARNING) return status;
     }
     /* raw->black can change in ufraw_subtract_darkframe() so we must
      * calculate uf->rgbMax first */
@@ -495,6 +613,13 @@ int ufraw_load_raw(ufraw_data *uf)
     } else {
 	uf->conf->ExposureNorm = 0;
     }
+    /* Foveon image dimensions are knows only after load_raw()*/
+    dcraw_image_dimensions(raw, uf->conf->orientation,
+	    &uf->initialHeight, &uf->initialWidth);
+    if (uf->conf->CropX2 > uf->initialWidth)
+	uf->conf->CropX2 = uf->initialWidth;
+    if (uf->conf->CropY2 > uf->initialHeight)
+	uf->conf->CropY2 = uf->initialHeight;
     /* chanMul[0]<0 signals that we need to recalculate the WB */
     if (uf->conf->chanMul[0]<0) ufraw_set_wb(uf);
     else {
@@ -516,50 +641,86 @@ int ufraw_load_raw(ufraw_data *uf)
 void ufraw_close(ufraw_data *uf)
 {
     dcraw_close(uf->raw);
+    g_free(uf->unzippedBuf);
     g_free(uf->raw);
+    /* We cannot free uf->conf since it might be accessed after the close */
+//    g_free(uf->conf);
     g_free(uf->exifBuf);
     g_free(uf->image.image);
+    int i;
+    for (i=ufraw_first_phase+1; i<ufraw_phases_num; i++)
+	g_free(uf->Images[i].buffer);
     g_free(uf->thumb.buffer);
     developer_destroy(uf->developer);
     g_free(uf->RawLumHistogram);
     ufraw_message(UFRAW_CLEAN, NULL);
 }
 
-/* Convert rawImage to standard rgb image */
-int ufraw_convert_image(ufraw_data *uf)
+int ufraw_convert_image_init(ufraw_data *uf)
 {
-    int c;
     dcraw_data *raw = uf->raw;
-    dcraw_image_data final;
-    int shrink = 1;
 
-//    preview_progress(uf->widget, _("Loading image"), 0.1);
     developer_prepare(uf->developer, uf->conf,
-	    uf->rgbMax, uf->rgb_cam, uf->colors, uf->useMatrix, FALSE);
+	    uf->rgbMax, uf->rgb_cam, uf->colors, uf->useMatrix, file_developer);
     /* We can do a simple interpolation in the following cases:
      * We shrink by an integer value.
      * If pixel_aspect<1 (e.g. NIKON D1X) shrink must be at least 4.
      * Wanted size is smaller than raw size (size is after a raw->shrink).
      * There are no filters (Foveon). */
+    uf->ConvertShrink = 1;
     if ( uf->conf->interpolation==half_interpolation ||
          ( uf->conf->size==0 && uf->conf->shrink>1 ) ||
          ( uf->conf->size>0 &&
 	   uf->conf->size<=MAX(raw->raw.height, raw->raw.width) ) ||
-	 ( raw->filters==0 ) ) {
+	 !uf->HaveFilters ) {
 	if (uf->conf->size==0 && uf->conf->shrink>1 &&
 		(int)(uf->conf->shrink*raw->pixel_aspect)%2==0)
-	    shrink = uf->conf->shrink * raw->pixel_aspect;
+	    uf->ConvertShrink = uf->conf->shrink * raw->pixel_aspect;
 	else if (uf->conf->interpolation==half_interpolation)
-	    shrink = 2;
-	else if (raw->filters!=0)
-	    shrink = 2;
-        dcraw_finalize_shrink(&final, raw, shrink);
+	    uf->ConvertShrink = 2;
+	else if ( uf->HaveFilters)
+	    uf->ConvertShrink = 2;
+    }
+    return UFRAW_SUCCESS;
+}
+
+int ufraw_convert_image(ufraw_data *uf)
+{
+    ufraw_convert_image_init(uf);
+    ufraw_convert_image_first_phase(uf);
+    if (uf->ConvertShrink>1) {
+	dcraw_data *raw = uf->raw;
+	dcraw_image_data final;
+        final.height = uf->image.height;
+        final.width = uf->image.width;
+        final.image = uf->image.image;
+	/* Scale threshold according to shrink factor, as the effect of
+	 * neighbouring pixels decays about exponentially with distance. */
+	float threshold = uf->conf->threshold * exp(-(uf->ConvertShrink/2.0-1));
+	dcraw_wavelet_denoise_shrinked(&final, raw, threshold);
+    }
+    return UFRAW_SUCCESS;
+}
+
+/* This is the part of the conversion which is not supported by
+ * ufraw_convert_image_area() */
+int ufraw_convert_image_first_phase(ufraw_data *uf)
+{
+    int status, c;
+    dcraw_data *raw = uf->raw;
+    dcraw_image_data final;
+
+    if ( uf->ConvertShrink>1 || !uf->HaveFilters ) {
+        dcraw_finalize_shrink(&final, raw, uf->ConvertShrink);
 
         uf->image.height = final.height;
         uf->image.width = final.width;
 	g_free(uf->image.image);
         uf->image.image = final.image;
     } else {
+	if ( (status=dcraw_wavelet_denoise(raw,
+		uf->conf->threshold))!=DCRAW_SUCCESS )
+	    return status;
         dcraw_finalize_interpolate(&final, raw, uf->conf->interpolation,
 		uf->developer->rgbWB);
 	uf->developer->rgbMax = uf->developer->max;
@@ -569,23 +730,172 @@ int ufraw_convert_image(ufraw_data *uf)
         uf->image.image = final.image;
     }
     dcraw_image_stretch(&final, raw->pixel_aspect);
-    if (uf->conf->size==0 && uf->conf->shrink>1)
+    if (uf->conf->size==0 && uf->conf->shrink>1) {
 	dcraw_image_resize(&final,
-		shrink*MAX(final.height, final.width)/uf->conf->shrink);
+	    uf->ConvertShrink*MAX(final.height, final.width)/uf->conf->shrink);
+	uf->ConvertShrink = uf->conf->shrink;
+    }
     if (uf->conf->size>0) {
         if ( uf->conf->size>MAX(final.height, final.width) ) {
             ufraw_message(UFRAW_ERROR, _("Can not downsize from %d to %d."),
                     MAX(final.height, final.width), uf->conf->size);
         } else {
+	    uf->ConvertShrink = MAX(final.height, final.width)/uf->conf->size;
             dcraw_image_resize(&final, uf->conf->size);
 	}
     }
-//    preview_progress(uf->widget, _("Loading image"), 0.4);
     uf->image.image = final.image;
-    dcraw_flip_image(&final, raw->flip);
+    dcraw_flip_image(&final, uf->conf->orientation);
     uf->image.height = final.height;
     uf->image.width = final.width;
-//    preview_progress(uf->widget, _("Loading image"), 0.5);
+
+    ufraw_image_data *FirstImage = &uf->Images[ufraw_first_phase];
+    FirstImage->height = uf->image.height;
+    FirstImage->width = uf->image.width;
+    FirstImage->depth = sizeof(dcraw_image_type);
+    FirstImage->rowstride = FirstImage->width * FirstImage->depth;
+    FirstImage->buffer = (guint8 *)uf->image.image;
+
+    return UFRAW_SUCCESS;
+}
+
+int ufraw_convert_image_init_phase(ufraw_data *uf)
+{
+    ufraw_image_data *FirstImage = &uf->Images[ufraw_first_phase];
+    ufraw_image_data *img;
+    if ( uf->conf->threshold>0 ) {
+	img = &uf->Images[ufraw_denoise_phase];
+	img->height = FirstImage->height;
+	img->width = FirstImage->width;
+	img->depth = sizeof(dcraw_image_type);
+	img->rowstride = img->width * img->depth;
+	img->buffer = g_realloc(img->buffer, img->height * img->rowstride);
+    }
+    img = &uf->Images[ufraw_final_phase];
+    img->height = FirstImage->height;
+    img->width = FirstImage->width;
+    img->depth = 3;
+    img->rowstride = img->width * img->depth;
+    img->buffer = g_realloc(img->buffer, img->height * img->rowstride);
+
+    return UFRAW_SUCCESS;
+}
+
+int ufraw_convert_image_area(ufraw_data *uf,
+	int x, int y, int width, int height, UFRawPhase fromPhase)
+{
+    dcraw_data *raw = uf->raw;
+    ufraw_image_data in = uf->Images[ufraw_first_phase];
+    int yy;
+
+    if ( fromPhase<=ufraw_denoise_phase && uf->conf->threshold>0 ) {
+	dcraw_image_data tmp;
+	/* With shrink==2 the border should be 16 pixels */
+	int border = 16 * 2 / uf->ConvertShrink;
+	int bx = MAX(x-border, 0);
+	int by = MAX(y-border, 0);
+	tmp.width = MIN(width + x-bx + border, in.width - bx);
+	if ( tmp.width<16 ) {
+	    bx = bx + tmp.width - 16; // We assume in.width>16
+	    tmp.width = 16;
+	}
+	tmp.height = MIN(height + y-by + border, in.height - by);
+	if ( tmp.height<16 ) {
+	    by = by + tmp.height - 16; // We assume in.height>16
+	    tmp.height = 16;
+	}
+	tmp.image = g_new(dcraw_image_type, tmp.height * tmp.width);
+	for (yy=0; yy<tmp.height; yy++)
+	    memcpy(tmp.image[yy*tmp.width],
+		    in.buffer + ((by+yy)*in.width + bx)*in.depth,
+		    tmp.width * in.depth);
+	/* Scale threshold according to shrink factor, as the effect of
+	 * neighbouring pixels decays about exponentially with distance. */
+	float threshold = uf->conf->threshold * exp(-(uf->ConvertShrink/2.0-1));
+	dcraw_wavelet_denoise_shrinked(&tmp, raw, threshold);
+	ufraw_image_data denoise = uf->Images[ufraw_denoise_phase];
+	for (yy=0; yy<height; yy++)
+	    memcpy(denoise.buffer + ((y+yy)*denoise.width + x)*denoise.depth,
+		    tmp.image[(yy+y-by)*width+x-bx], width*denoise.depth);
+	g_free(tmp.image);
+    }
+    if ( uf->conf->threshold>0 ) in = uf->Images[ufraw_denoise_phase];
+    guint16 *pixtmp = g_new(guint16, width*3);
+    ufraw_image_data out = uf->Images[ufraw_final_phase];
+    for (yy=y; yy<y+height; yy++) {
+	develope(out.buffer + (yy*out.width + x)*out.depth,
+		(void*)in.buffer + (yy*in.width + x)*in.depth,
+		uf->developer, 8, pixtmp, width);
+    }
+    g_free(pixtmp);
+    return UFRAW_SUCCESS;
+}
+
+int ufraw_flip_image_buffer(ufraw_image_data *img, int flip)
+{
+    /* Following code was copied from dcraw's flip_image()
+     * and modified to work with any pixel depth. */
+    int base, dest, next, row, col;
+    guint8 *image = img->buffer;
+    int height = img->height;
+    int width = img->width;
+    int depth = img->depth;
+    int size = height * width;
+    guint8 hold[8];
+    unsigned *flag = g_new0(unsigned, (size+31) >> 5);
+    for (base = 0; base < size; base++) {
+	if (flag[base >> 5] & (1 << (base & 31)))
+	    continue;
+	dest = base;
+	memcpy(hold, image+base*depth, depth);
+	while (1) {
+	    if (flip & 4) {
+		row = dest % height;
+		col = dest / height;
+	    } else {
+		row = dest / width;
+		col = dest % width;
+	    }
+	    if (flip & 2)
+		row = height - 1 - row;
+	    if (flip & 1)
+		col = width - 1 - col;
+	    next = row * width + col;
+	    if (next == base) break;
+	    flag[next >> 5] |= 1 << (next & 31);
+	    memcpy(image+dest*depth, image+next*depth, depth);
+	    dest = next;
+	}
+	memcpy(image+dest*depth, hold, depth);
+    }
+    g_free (flag);
+    if (flip & 4) {
+	img->height = width;
+	img->width = height;
+	img->rowstride = height * depth;
+    }
+    return UFRAW_SUCCESS;
+}
+
+int ufraw_flip_image(ufraw_data *uf, int flip)
+{
+    const int flipMatrix[8][8] = {
+	{ 0, 1, 2, 3, 4, 5, 6, 7 }, /* No flip */
+	{ 1, 0, 3, 2, 5, 4, 7, 6 }, /* Flip horizontal */
+	{ 2, 3, 0, 1, 6, 7, 4, 5 }, /* Flip vertical */
+	{ 3, 2, 1, 0, 7, 6, 5, 4 }, /* Rotate 180 */
+	{ 4, 6, 5, 7, 0, 2, 1, 3 }, /* Flip over diagonal "\" */
+	{ 5, 7, 4, 6, 1, 3, 0, 2 }, /* Rotate 270 */
+	{ 6, 4, 7, 5, 2, 0, 3, 1 }, /* Rotate 90 */
+	{ 7, 5, 6, 4, 3, 1, 2, 0 }  /* Flip over diagonal "/" */
+    };
+    uf->conf->orientation = flipMatrix[uf->conf->orientation][flip];
+
+    ufraw_flip_image_buffer(&uf->Images[ufraw_first_phase], flip);
+    if ( uf->conf->threshold>0 )
+	ufraw_flip_image_buffer(&uf->Images[ufraw_denoise_phase], flip);
+    ufraw_flip_image_buffer(&uf->Images[ufraw_final_phase], flip);
+ 
     return UFRAW_SUCCESS;
 }
 
@@ -658,12 +968,12 @@ int ufraw_set_wb(ufraw_data *uf)
 		}
 	    }
 	}
-        gint64 sum;
         for (c=0; c<uf->colors; c++) {
-            sum = 0;
+	    gint64 sum = 0;
             for (i=0; i<uf->rgbMax+1; i++)
                 sum += (gint64)i*histogram[i][c];
-            uf->conf->chanMul[c] = 1.0/sum;
+	    if (sum==0) uf->conf->chanMul[c] = 1.0;
+            else uf->conf->chanMul[c] = 1.0/sum;
         }
 	g_free(histogram);
 	uf->conf->WBTuning = 0;
@@ -829,7 +1139,7 @@ void ufraw_auto_expose(ufraw_data *uf)
     if ( uf->conf->ExposureNorm>0 )
 	uf->conf->exposure = -log(1.0*uf->rgbMax/uf->conf->ExposureNorm)/log(2);
     developer_prepare(uf->developer, uf->conf,
-	    uf->rgbMax, uf->rgb_cam, uf->colors, uf->useMatrix, TRUE);
+	    uf->rgbMax, uf->rgb_cam, uf->colors, uf->useMatrix, auto_developer);
     /* Find the grey value that gives 99% luminosity */
     double maxChan = 0;
     for (c=0; c<uf->colors; c++) maxChan = MAX(uf->conf->chanMul[c], maxChan);
@@ -870,7 +1180,7 @@ void ufraw_auto_black(ufraw_data *uf)
 
     /* Reset the luminosityCurve */
     developer_prepare(uf->developer, uf->conf,
-	    uf->rgbMax, uf->rgb_cam, uf->colors, uf->useMatrix, TRUE);
+	    uf->rgbMax, uf->rgb_cam, uf->colors, uf->useMatrix, auto_developer);
     /* Calculate the black point */
     ufraw_build_raw_luminosity_histogram(uf);
     stop = uf->RawLumCount/256/4;
@@ -905,7 +1215,7 @@ void ufraw_auto_curve(ufraw_data *uf)
 
     CurveDataReset(curve);
     developer_prepare(uf->developer, uf->conf,
-	    uf->rgbMax, uf->rgb_cam, uf->colors, uf->useMatrix, TRUE);
+	    uf->rgbMax, uf->rgb_cam, uf->colors, uf->useMatrix, auto_developer);
     /* Calculate curve points */
     ufraw_build_raw_luminosity_histogram(uf);
     stop = uf->RawLumCount/256/4;
